#! /usr/bin/env python


import numpy as np
import itertools


INTERIOR_NODE = 0
FIXED_VALUE_BOUNDARY = 1
FIXED_GRADIENT_BOUNDARY = 2
TRACKS_CELL_BOUNDARY = 3
INACTIVE_BOUNDARY = 4

BAD_INDEX_VALUE = np.iinfo(np.int).max

from landlab.utils import count_repeated_values


def node_count(shape):
    """
    The total number of nodes in a structured grid with dimensions given
    by the tuple, *shape*. Where *shape* is the number of node rows and
    node columns.

    >>> node_count((3, 4))
    12
    """
    assert(len(shape) == 2)
    return shape[0] * shape[1]


def interior_node_count(shape):
    """
    Return the count of the number of interior nodes of a structured grid
    of dimensions, *shape*.

    >>> node_count((2, 4))
    8
    >>> interior_node_count((2, 4))
    0
    >>> interior_node_count((1, 4))
    0
    >>> interior_node_count((3, 4))
    2
    """
    assert(len(shape) == 2)
    try:
        assert(np.min(shape) > 2)
    except AssertionError:
        return 0
    else:
        return (shape[0] - 2) * (shape[1] - 2)


def cell_count(shape):
    """
    The total number of cells in a structured grid with dimensions, *shape*.
    Where *shape* is a tuple that gives the dimensions of the grid as number
    of rows of nodes followed by number of columns of nodes.

    >>> cell_count((3, 4))
    2
    >>> cell_count((1, 4))
    0
    """
    assert(len(shape) == 2)

    if np.min(shape) > 2:
        return (shape[0] - 2) * (shape[1] - 2)
    else:
        return 0


def active_cell_count(shape):
    """
    Number of active cells. By default, all cells are active so this is
    the same as cell_count.
    """
    return cell_count(shape)


def active_link_count(shape):
    """
    Number of active links in a structured grid with dimensions, *shape*.
    A link is active if it connects to at least one active node.

    >>> link_count((3, 2))
    7
    >>> active_link_count((3, 2))
    0
    >>> active_link_count((3, 4))
    7
    """
    assert(len(shape) == 2)
    try:
        assert(np.min(shape) > 2)
    except AssertionError:
        return 0
    else:
        return 2 * shape[0] * shape[1] - 3 * (shape[0] + shape[1]) + 4


def link_count(shape):
    """
    Total (active and inactive) number of links in a structured grid with
    dimensions, *shape*. This is the number of to-links and from-links, not
    the total of the two.

    >>> link_count((3,2))
    7
    """
    assert(len(shape) == 2)
    return shape[1] * (shape[0] - 1) + shape[0] * (shape[1] - 1)


def vertical_link_count(shape):
    assert(len(shape) == 2)
    return (shape[0] - 1) * shape[1]


def horizontal_link_count(shape):
    assert(len(shape) == 2)
    return shape[0] * (shape[1] - 1)


def boundary_cell_count(shape):
    """
    Number of cells that are on the boundary of a structured grid with
    dimensions, *shape*. In fact, cells centered on boundary nodes are not
    really cells. If they were, though, this is how many there would be.

    >>> boundary_cell_count((3, 4))
    10
    """
    assert(len(shape) == 2)
    return 2 * (shape[0] - 2) + 2 * (shape[1] - 2) + 4


def interior_cell_count(shape):
    """
    Number of interior cells. Since cells are only defined on interior nodes,
    this is the same as cell_count.
    """
    return cell_count(shape)


def face_count(shape):
    """
    Total number of faces in a structured grid with dimensions, *shape*. Each
    cell has four faces, and shared faces only count once.

    >>> face_count((3, 4))
    7
    """
    assert(len(shape) == 2)
    if np.min(shape) > 2:
        return ((shape[0] - 1) * (shape[1] - 2) +
                (shape[0] - 2) * (shape[1] - 1))
    else:
        return 0


def active_face_count(shape):
    """
    Total number of active faces in a structured grid with dimensions,
    *shape*. Each cell has four faces, and shared faces only count once.
    An active face is one that has a corresponing active link.

    >>> active_face_count((3, 4))
    7
    """
    return face_count(shape)


def top_index_iter(shape):
    """
    Iterator for the top boundary indices of a structured grid.
    """
    return xrange(shape[1] * (shape[0] - 1), shape[0] * shape[1])


def bottom_index_iter(shape):
    """
    Iterator for the bottom boundary indices of a structured grid.
    """
    return xrange(0, shape[1])


def left_index_iter(shape):
    """
    Iterator for the left boundary indices of a structured grid.
    """
    return xrange(0, shape[0] * shape[1], shape[1])


def right_index_iter(shape):
    """
    Iterator for the right boundary indices of a structured grid.
    """
    return xrange(shape[1] - 1, shape[0] * shape[1], shape[1])


def left_right_iter(shape, *args):
    """
    Iterator for the left and right boundary indices of a structured grid.
    This iterates over the indices in order rather than iterating all of
    the left boundary and then all of the right boundary.

    >>> import numpy as np
    >>> np.fromiter(left_right_iter((4, 3)), dtype=np.int)
    array([ 0,  2,  3,  5,  6,  8,  9, 11])
    >>> np.fromiter(left_right_iter((4, 3), 2), dtype=np.int)
    array([0, 2, 3, 5])
    >>> np.fromiter(left_right_iter((4, 3), 2, 4), dtype=np.int)
    array([ 6,  8,  9, 11])
    >>> np.fromiter(left_right_iter((4, 3), 1, 4, 2), dtype=np.int)
    array([ 3,  5,  9, 11])
    """
    if len(args) == 0:
        iter_rows = xrange(0, shape[0], 1)
    elif len(args) == 1:
        iter_rows = xrange(0, args[0], 1)
    elif len(args) == 2:
        iter_rows = xrange(args[0], args[1], 1)
    elif len(args) == 3:
        iter_rows = xrange(args[0], args[1], args[2])

    for row in iter_rows:
        yield row * shape[1]
        yield row * shape[1] + shape[1] - 1


def bottom_top_iter(shape):
    """
    Iterates of the bottom indices and then the top indices of a structured
    grid.

    >>> import numpy as np
    >>> np.fromiter(bottom_top_iter((4, 3)), dtype=np.int)
    array([ 0,  1,  2,  9, 10, 11])
    """
    return itertools.chain(bottom_index_iter(shape),
                           top_index_iter(shape))


def boundary_iter(shape):
    """
    Iterates over all of the boundary node indices of a structured grid in
    order.

    >>> import numpy as np
    >>> np.fromiter(boundary_iter((4, 3)), dtype=np.int)
    array([ 0,  1,  2,  3,  5,  6,  8,  9, 10, 11])
    """
    return itertools.chain(bottom_index_iter(shape),
                           left_right_iter(shape, 1, shape[0] - 1),
                           top_index_iter(shape))


def boundary_nodes(shape):
    """
    An array of the indices of the boundary nodes.

    >>> boundary_nodes((3, 4))
    array([ 0,  1,  2,  3,  4,  7,  8,  9, 10, 11])
    """
    return np.fromiter(boundary_iter(shape), dtype=np.int)
    
def corners(shape):
    """
    An array of the indices of the grid corner nodes.
    """
    return np.array([0, shape[1]-1, shape[1]*(shape[0]-1), shape[1]*shape[0]-1])

def bottom_edge_node_ids(shape):
    return np.fromiter(bottom_index_iter(shape), dtype=np.int)


def top_edge_node_ids(shape):
    return np.fromiter(top_index_iter(shape), dtype=np.int)


def left_edge_node_ids(shape):
    return np.fromiter(left_index_iter(shape), dtype=np.int)


def right_edge_node_ids(shape):
    return np.fromiter(right_index_iter(shape), dtype=np.int)


def interior_iter(shape):
    """
    Iterate over the interior nodes of a structured grid.

    >>> import numpy as np
    >>> np.fromiter(interior_iter((4, 3)), dtype=np.int)
    array([4, 7])
    """
    interiors = []
    interiors_per_row = shape[1] - 2
    for row in xrange(shape[1] + 1, shape[1] * (shape[0] - 1), shape[1]):
        interiors.append(xrange(row , row + interiors_per_row))
    return itertools.chain(*interiors)


def interior_nodes(shape):
    return np.fromiter(interior_iter(shape), dtype=np.int)


def node_coords(shape, *args):
    """
    Get x, y coordinates for nodes in a structured grid with dimensions,
    *shape*. Use the optional argument *spacing* to give the spacing in each
    dimension, and *origin* the start of the coordinates in each dimension.

    >>> (cols, rows) = node_coords((3, 2))
    >>> rows
    array([ 0.,  0.,  1.,  1.,  2.,  2.])
    >>> cols
    array([ 0.,  1.,  0.,  1.,  0.,  1.])
    """
    try:
        spacing = args[0]
    except IndexError:
        spacing = np.ones(len(shape), dtype=np.float)
    else:
        assert(len(spacing) == len(shape))

    try:
        origin = args[1]
    except IndexError:
        origin = np.zeros(len(shape), dtype=np.float)
    else:
        assert(len(origin) == len(origin))

    node_count = np.prod(shape)

    row_y = np.arange(shape[0]) * spacing[0] + origin[0] 
    col_x = np.arange(shape[1]) * spacing[1] + origin[1]

    (node_x, node_y) = np.meshgrid(col_x, row_y)

    node_x.shape = (node_count, )
    node_y.shape = (node_count, )

    return (node_x, node_y)


def active_cell_index(shape):
    """
    Ordered indices of the active cells of a structured grid.
    """
    return np.arange(active_cell_count(shape))


def active_cell_node(shape):
    """
    Indices of the nodes belonging to each cell. Since all cells are active,
    this is the same as node_index_at_cells.

    >>> node_index_at_cells((4,3))
    array([4, 7])
    """
    return node_index_at_cells(shape)


def active_cell_index_at_nodes(shape, boundary_node_index=BAD_INDEX_VALUE):
    """
    Indices of the cells associated with the nodes of the structured grid.
    For nodes that don't have a cell (that is, boundary nodes) set indices
    to BAD_INDEX_VALUE. Use the *boundary_node_index* keyword to change
    the value of indices to boundary nodes.

    >>> active_cell_index_at_nodes((3, 4), boundary_node_index=-1) # doctest: +NORMALIZE_WHITESPACE
    array([-1, -1, -1, -1,
           -1,  0,  1, -1,
           -1, -1, -1, -1])
    """
    n_nodes = node_count(shape)

    node_ids = np.empty(n_nodes, dtype=np.int)

    node_ids[boundary_nodes(shape)] = boundary_node_index
    node_ids[interior_nodes(shape)] = np.arange(interior_node_count(shape))

    return node_ids


def node_index_at_cells(shape):
    """
    Indices of the nodes belonging to each cell.

    >>> node_index_at_cells((4, 3))
    array([4, 7])
    """
    node_ids = np.arange(node_count(shape))
    node_ids.shape = shape

    cell_node = node_ids[1:-1, 1:-1].copy()
    cell_node.shape = ((shape[0] - 2) * (shape[1] - 2), )

    return cell_node


def node_index_at_link_ends(shape):
    node_ids = np.arange(np.prod(shape))
    node_ids.shape = shape

    return (node_index_at_link_head(node_ids),
            node_index_at_link_tail(node_ids))


def inlink_index_at_node(shape):
    return inlinks(shape, return_count=False)


def outlink_index_at_node(shape):
    return outlinks(shape, return_count=False)


def node_index_at_link_tail(node_ids):
    vertical_links = node_ids[1:, :]
    horizontal_links = node_ids[:, 1:]
    return np.concatenate((vertical_links.flat, horizontal_links.flat))


def node_index_at_link_head(node_ids):
    vertical_links = node_ids[:-1, :]
    horizontal_links = node_ids[:, :-1]
    return np.concatenate((vertical_links.flat, horizontal_links.flat))


def face_index_at_links(shape, actives=None,
                        inactive_link_index=BAD_INDEX_VALUE):
    """
    Returns an array that maps link ids to face ids. For inactive links,
    which do not have associated faces, set their ids to
    *inactive_link_index*. Use the *actives* keyword to specify an array that
    contains the ids of all active links in the grid. The default assumes
    that only the perimeter nodes are inactive.


    >>> faces = face_index_at_links((3, 4), inactive_link_index=-1)
    >>> faces # doctest: +NORMALIZE_WHITESPACE
    array([-1,  0,  1, -1, -1,  2,  3,
           -1, -1, -1, -1,  4,  5,  6, -1, -1, -1])
    
    """
    if actives is None:
        actives = active_links(shape)

    num_links = link_count(shape)

    link_faces = np.empty(num_links, dtype=np.int)
    link_faces.fill(inactive_link_index)
    link_faces[actives] = np.arange(len(actives))

    return link_faces


def node_status(shape, boundary_status=FIXED_VALUE_BOUNDARY):
    """
    The statuses of the nodes in a structured grid with dimensions, *shape*.
    Use the *boundary_status* keyword to specify the status of the top,
    bottom, left and right boundary nodes.
    """
    status = np.empty(np.prod(shape), dtype=np.int8)

    status[interior_nodes(shape)] = INTERIOR_NODE
    status[boundary_nodes(shape)] = boundary_status

    return status


def active_links(shape, node_status_array=None, link_nodes=None):
    """
    >>> active_links((3, 4))
    array([ 1,  2,  5,  6, 11, 12, 13])
    """
    if node_status_array is None:
        node_status_array = node_status(shape)

    if link_nodes is None:
        (link_from_node, link_to_node) = node_index_at_link_ends(shape)
    else:
        (link_from_node, link_to_node) = link_nodes

    from_node_status = node_status_array[link_from_node]
    to_node_status = node_status_array[link_to_node]

    active_links = (((from_node_status == INTERIOR_NODE) & ~
                     (to_node_status == INACTIVE_BOUNDARY)) |
                    ((to_node_status == INTERIOR_NODE) & ~
                     (from_node_status == INACTIVE_BOUNDARY)))

    (active_links, ) = np.where(active_links)

    return active_links


def active_face_index(shape):
    return np.arange(active_face_count(shape))


def inlinks(shape, tonodes=None, return_count=True):
    links = np.vstack((south_links(shape), west_links(shape)))
    links.shape = (2, node_count(shape))
    return links


def outlinks(shape, tonodes=None, return_count=True):
    links = np.vstack((north_links(shape), east_links(shape)))
    links.shape = (2, node_count(shape))
    return links


def active_inlinks(shape, node_status=None, return_count=True):
    links = np.vstack((active_south_links(shape, node_status),
                       active_west_links(shape, node_status)))
    links.shape = (2, node_count(shape))
    return links


def active_outlinks(shape, node_status=None, return_count=True):
    links = np.vstack((active_north_links(shape, node_status),
                       active_east_links(shape, node_status)))
    links.shape = (2, node_count(shape))
    return links


def vertical_link_ids(shape):
    link_ids = np.arange(0, vertical_link_count(shape), dtype=np.int)
    link_ids.shape = (shape[0] - 1, shape[1])
    return link_ids


def horizontal_link_ids(shape):
    link_id_offset = vertical_link_count(shape)
    link_ids = np.arange(link_id_offset,
                         link_id_offset + horizontal_link_count(shape),
                         dtype=np.int)
    link_ids.shape = (shape[0], shape[1] - 1)
    return link_ids


def vertical_active_link_count(shape, node_status=None):
    if node_status is not None:
        is_inactive = (node_status == 0)
        is_inactive.shape = shape

        inactive_outlinks = is_inactive[:-1, 1:-1]
        inactive_inlinks = is_inactive[1:, 1:-1]
        inactive_links = inactive_outlinks | inactive_inlinks

        return (shape[0] - 1) * (shape[1] - 2) - np.sum(inactive_links.flat)
    else:
        return (shape[0] - 1) * (shape[1] - 2)


def horizontal_active_link_count(shape, node_status=None):
    if node_status is not None:
        is_inactive = (node_status == 0)
        is_inactive.shape = shape

        inactive_outlinks = is_inactive[1:-1, :-1]
        inactive_inlinks = is_inactive[1:-1, 1:]
        inactive_links = inactive_outlinks | inactive_inlinks

        return (shape[0] - 2) * (shape[1] - 1) - np.sum(inactive_links.flat)
    else:
        return (shape[0] - 2) * (shape[1] - 1)


def vertical_inactive_link_mask(shape, node_status):
    is_inactive = (node_status == 0)
    is_inactive.shape = shape

    inactive_outlinks = is_inactive[:-1, 1:-1]
    inactive_inlinks = is_inactive[1:, 1:-1]
    return inactive_outlinks | inactive_inlinks


def horizontal_inactive_link_mask(shape, node_status):
    is_inactive = (node_status == 0)
    is_inactive.shape = shape

    inactive_outlinks = is_inactive[1:-1, :-1]
    inactive_inlinks = is_inactive[1:-1, 1:]
    return inactive_outlinks | inactive_inlinks


#def vertical_active_link_ids(shape):
#    link_ids = np.arange(vertical_active_link_count(shape), dtype=np.int)
#    link_ids.shape = (shape[0] - 1, shape[1] - 2)
#    return link_ids


def vertical_active_link_ids(shape, node_status=None):
    if node_status is None:
        link_ids = np.arange(vertical_active_link_count(shape), dtype=np.int)
        #link_ids.shape = (shape[0] - 1, shape[1] - 2)
        #return link_ids
    else:
        inactive_links = vertical_inactive_link_mask(shape, node_status)
        inactive_links.shape = (inactive_links.size, )
        active_link_count = inactive_links.size - np.sum(inactive_links)

        link_ids = np.empty(inactive_links.size)
        link_ids[inactive_links] = - 1
        link_ids[~ inactive_links] = np.arange(active_link_count, dtype=int)

    link_ids.shape = (shape[0] - 1, shape[1] - 2)
    return link_ids


def horizontal_active_link_ids(shape, node_status=None):
    if node_status is None:
        link_id_offset = vertical_active_link_count(shape)
        link_ids = np.arange(
            link_id_offset,
            link_id_offset + horizontal_active_link_count(shape),
            dtype=np.int)
    else:
        link_id_offset = vertical_active_link_count(shape,
                                                    node_status=node_status)
        inactive_links = horizontal_inactive_link_mask(shape, node_status)
        inactive_links.shape = (inactive_links.size, )
        active_link_count = inactive_links.size - np.sum(inactive_links)

        link_ids = np.empty(inactive_links.size)
        link_ids[inactive_links] = - 1
        link_ids[~ inactive_links] = np.arange(
            link_id_offset, link_id_offset + active_link_count,
            dtype=np.int)
    link_ids.shape = (shape[0] - 2, shape[1] - 1)
    return link_ids


def west_links(shape):
    """
    >>> west_links((3, 4))
    array([[-1,  8,  9, 10],
           [-1, 11, 12, 13],
           [-1, 14, 15, 16]])
    """
    link_ids = horizontal_link_ids(shape)
    link_ids.shape = (shape[0], shape[1] - 1)
    return np.hstack((- np.ones((shape[0], 1), dtype=np.int), link_ids))


def north_links(shape):
    """
    >>> north_links((3, 4))
    array([[ 0,  1,  2,  3],
           [ 4,  5,  6,  7],
           [-1, -1, -1, -1]])
    """
    link_ids = vertical_link_ids(shape)
    link_ids.shape = (shape[0] - 1, shape[1])
    return np.vstack((link_ids, - np.ones((1, shape[1]), dtype=np.int)))


def south_links(shape):
    """
    >>> south_links((3, 4))
    array([[-1, -1, -1, -1],
           [ 0,  1,  2,  3],
           [ 4,  5,  6,  7]])
    """
    link_ids = vertical_link_ids(shape)
    link_ids.shape = (shape[0] - 1, shape[1])
    return np.vstack((- np.ones((1, shape[1]), dtype=np.int), link_ids))

def east_links(shape):
    """
    >>> east_links((3, 4))
    array([[ 8,  9, 10, -1],
           [11, 12, 13, -1],
           [14, 15, 16, -1]])
    """
    link_ids = horizontal_link_ids(shape)
    link_ids.shape = (shape[0], shape[1] - 1)
    return np.hstack((link_ids, - np.ones((shape[0], 1), dtype=int)))


def active_north_links(shape, node_status=None):
    """
    >>> active_north_links((3, 4))
    array([[-1,  0,  1, -1],
           [-1,  2,  3, -1],
           [-1, -1, -1, -1]])
    """
    active_north_links = np.empty(shape, dtype=int)
    try:
        links = vertical_active_link_ids(shape, node_status=node_status)
    except ValueError:
        pass
    links.shape = (shape[0] - 1, shape[1] - 2)
    active_north_links[:-1, 1:-1] = links
    active_north_links[:, (0, -1)] = -1
    active_north_links[-1, :] = -1

    return active_north_links


def active_south_links(shape, node_status=None):
    """
    >>> active_south_links((3, 4))
    array([[-1, -1, -1, -1],
           [-1,  0,  1, -1],
           [-1,  2,  3, -1]])
    """
    active_south_links = np.empty(shape, dtype=int)
    links = vertical_active_link_ids(shape, node_status=node_status)
    links.shape = (shape[0] - 1, shape[1] - 2)
    active_south_links[1:, 1:-1] = links
    active_south_links[:, (0, -1)] = -1
    active_south_links[0, :] = -1

    return active_south_links


def active_west_links(shape, node_status=None):
    """
    >>> active_west_links((3, 4))
    array([[-1, -1, -1, -1],
           [-1,  4,  5,  6],
           [-1, -1, -1, -1]])
    """
    active_west_links = np.empty(shape, dtype=int)
    try:
        active_west_links[1:-1, 1:] = horizontal_active_link_ids(
            shape, node_status=node_status)
    except ValueError:
        pass
    active_west_links[(0, -1), :] = -1
    active_west_links[:, 0] = -1

    return active_west_links


def active_east_links(shape, node_status=None):
    """
    >>> active_east_links((3, 4))
    array([[-1, -1, -1, -1],
           [ 4,  5,  6, -1],
           [-1, -1, -1, -1]])
    """
    active_east_links = np.empty(shape, dtype=int)
    active_east_links.fill(-999)
    try:
        active_east_links[1:-1, :-1] = horizontal_active_link_ids(
            shape, node_status=node_status)
    except ValueError:
        pass
    active_east_links[(0, -1), :] = -1
    active_east_links[:, -1] = -1

    return active_east_links


def outlink_count_per_node(shape):
    link_count = np.empty(shape, dtype=np.int)
    link_count[:-1, :-1] = 2
    link_count[-1, :-1] = 1
    link_count[:-1, -1] = 1
    link_count[-1, -1] = 0
    return np.ravel(link_count)


def inlink_count_per_node(shape):
    link_count = np.empty(shape, dtype=np.int)
    link_count[1:, 1:] = 2
    link_count[0, 1:] = 1
    link_count[1:, 0] = 1
    link_count[0, 0] = 0
    return np.ravel(link_count)


def active_outlink_count_per_node(shape):
    link_count = np.empty(shape, dtype=np.int)
    link_count[1:-1, 1:-1] = 2
    link_count[0, :] = 1
    link_count[-1, :] = 0
    link_count[:, 0] = 1
    link_count[:, -1] = 0

    link_count[0, 0] = 0
    link_count[-1, 0] = 0

    return np.ravel(link_count)


def active_inlink_count_per_node(shape):
    link_count = np.empty(shape, dtype=np.int)
    link_count[1:-1, 1:-1] = 2
    link_count[0, :] = 0
    link_count[-1, :] = 1
    link_count[:, 0] = 0
    link_count[:, -1] = 1

    link_count[0, -1] = 0
    link_count[-1, -1] = 0

    return np.ravel(link_count)


def setup_outlink_matrix(shape, fromnodes=None, return_count=True):
    links = outlinks(shape)
    if return_count:
        return (links, outlink_count_per_node(shape))
    else:
        return links


def setup_inlink_matrix(shape, tonodes=None, return_count=True):
    links = inlinks(shape)
    if return_count:
        return (links, inlink_count_per_node(shape))
    else:
        return links


def setup_active_outlink_matrix(shape, node_status=None, return_count=True):
    links = active_outlinks(shape, node_status=node_status)
    if return_count:
        return (links, active_outlink_count_per_node(shape))
    else:
        return links


def setup_active_inlink_matrix(shape, node_status=None, return_count=True):
    links = active_inlinks(shape, node_status=node_status)
    if return_count:
        return (links, active_inlink_count_per_node(shape))
    else:
        return links


def node_index_with_halo(shape, halo_indices=BAD_INDEX_VALUE):
    """
    >>> node_index_with_halo((2, 3), halo_indices=-1)
    array([[-1, -1, -1, -1, -1],
           [-1,  0,  1,  2, -1],
           [-1,  3,  4,  5, -1],
           [-1, -1, -1, -1, -1]])
    """
    shape_with_halo = np.array(shape) + 2

    ids = np.empty(shape_with_halo, dtype=np.int)

    (interiors, boundaries) = (interior_nodes(shape_with_halo),
                               boundary_nodes(shape_with_halo)) 

    ids.flat[interiors] = xrange(interior_node_count(shape_with_halo))
    ids.flat[boundaries] = halo_indices

    return ids


def cell_index_with_halo(shape, halo_indices=BAD_INDEX_VALUE,
                        inactive_indices=None):
    """
    >>> cell_index_with_halo((2, 3), halo_indices=-1)
    array([[-1, -1, -1, -1, -1],
           [-1,  0,  1,  2, -1],
           [-1,  3,  4,  5, -1],
           [-1, -1, -1, -1, -1]])

    >>> cell_index_with_halo((2, 3), halo_indices=-1, inactive_indices=-1)
    array([[-1, -1, -1, -1, -1],
           [-1, -1, -1, -1, -1],
           [-1, -1, -1, -1, -1],
           [-1, -1, -1, -1, -1]])
    """
    ids = node_index_with_halo(shape, halo_indices=halo_indices)
    if inactive_indices is not None:
        ids[:, (1, -2)] = inactive_indices
        ids[(1, -2), :] = inactive_indices

    return ids


def neighbor_node_array(shape, out_of_bounds=BAD_INDEX_VALUE, contiguous=True,
                        boundary_node_mask=None):
    """
    >>> neighbors = neighbor_node_array((2, 3), out_of_bounds=-1)
    >>> neighbors
    array([[ 1,  3, -1, -1],
           [ 2,  4,  0, -1],
           [-1,  5,  1, -1],
           [ 4, -1, -1,  0],
           [ 5, -1,  3,  1],
           [-1, -1,  4,  2]])
    >>> neighbors.flags['C_CONTIGUOUS']
    True
    >>> neighbors = neighbor_node_array((2, 3), out_of_bounds=-1, contiguous=False)
    >>> neighbors.flags['C_CONTIGUOUS']
    False
    """
    ids = node_index_with_halo(shape, halo_indices=out_of_bounds)

    neighbors = np.vstack((
        ids[1:shape[0] + 1, 2:].flat,
        ids[2:, 1:shape[1] + 1].flat,
        ids[1:shape[0] + 1, :shape[1]].flat,
        ids[:shape[0], 1:shape[1] + 1].flat,)).T

    if boundary_node_mask is not None:
        boundaries = np.empty(4, dtype=np.int)
        boundaries.fill(boundary_node_mask)
        neighbors[boundary_nodes(shape)] = boundaries

    if contiguous:
        return neighbors.copy()
    else:
        return neighbors


def neighbor_cell_array(shape, out_of_bounds=BAD_INDEX_VALUE, contiguous=True):
    """
    >>> neighbors = neighbor_cell_array((2, 3), out_of_bounds=-1)
    >>> neighbors
    array([], dtype=int64)

    >>> neighbors = neighbor_cell_array((3, 3), out_of_bounds=-1)
    >>> neighbors
    array([[-1, -1, -1, -1]])

    >>> neighbors = neighbor_cell_array((5, 4), out_of_bounds=-1)
    >>> neighbors # doctest: +NORMALIZE_WHITESPACE
    array([[ 1,  2, -1, -1], [-1,  3,  0, -1],
           [ 3,  4, -1,  0], [-1,  5,  2,  1],
           [ 5, -1, -1,  2], [-1, -1,  4,  3]])
    """
    if cell_count(shape) > 0:
        shape = np.array(shape) - 2
        ids = node_index_with_halo(shape, halo_indices=out_of_bounds)

        neighbors = np.vstack((
            ids[1:shape[0] + 1, 2:].flat,
            ids[2:, 1:shape[1] + 1].flat,
            ids[1:shape[0] + 1, :shape[1]].flat,
            ids[:shape[0], 1:shape[1] + 1].flat,)).T
        if contiguous:
            return neighbors.copy()
        else:
            return neighbors 
    else:
        return np.array([], dtype=np.int)


def diagonal_node_array(shape, out_of_bounds=BAD_INDEX_VALUE, contiguous=True,
                        boundary_node_mask=None):
    """
    Creates a list of IDs of the diagonal cells to each cell, as a 2D array. 
    Only interior cells are assigned neighbors; boundary cells get -1 for
    each neighbor.  The order of the diagonal cells is [topright, topleft,
    bottomleft, bottomright].
    
    NG didn't touch this, but she thinks this should be nodes, not cells.

    >>> diags = diagonal_node_array((2, 3), out_of_bounds=-1)
    >>> diags
    array([[ 4, -1, -1, -1],
           [ 5,  3, -1, -1],
           [-1,  4, -1, -1],
           [-1, -1, -1,  1],
           [-1, -1,  0,  2],
           [-1, -1,  1, -1]])
    >>> diags.flags['C_CONTIGUOUS']
    True
    >>> diags = diagonal_node_array((2, 3), out_of_bounds=-1, contiguous=False)
    >>> diags.flags['C_CONTIGUOUS']
    False
    """
    ids = node_index_with_halo(shape, halo_indices=out_of_bounds)

    diags = np.vstack((
        ids[2:, 2:].flat,
        ids[2:, :shape[1]].flat,
        ids[:shape[0], :shape[1]].flat,
        ids[:shape[0], 2:].flat,)).T

    if boundary_node_mask is not None:
        boundaries = np.empty(4, dtype=np.int)
        boundaries.fill(boundary_node_mask)
        diags[boundary_nodes(shape)] = boundaries

    if contiguous:
        return diags.copy()
    else:
        return diags


def diagonal_cell_array(shape, out_of_bounds=BAD_INDEX_VALUE, contiguous=True):
    """
    Construct a matrix of cell indices to each diagonally adjacent cell of a
    structured grid. If a cell does not have a diagonal neighbor, set the
    index for that neighbor to *out_of_bounds*.

    An grid without any cells returns an empty array.
    >>> diags = diagonal_cell_array((2, 3), out_of_bounds=-1)
    >>> diags
    array([], dtype=int64)

    A grid that has only one cell does not have any neighbors so all of its
    diagonals are set to *out_of_bounds*.
    >>> diags = diagonal_cell_array((3, 3), out_of_bounds=-1)
    >>> diags
    array([[-1, -1, -1, -1]])

    >>> diags = diagonal_cell_array((4, 4), out_of_bounds=-1)
    >>> diags # doctest: +NORMALIZE_WHITESPACE
    array([[ 3, -1, -1, -1], [-1,  2, -1, -1],
           [-1, -1, -1,  1], [-1, -1,  0, -1]])

    >>> diags = diagonal_cell_array((4, 5), out_of_bounds=-1)
    >>> diags # doctest: +NORMALIZE_WHITESPACE
    array([[ 4, -1, -1, -1], [ 5,  3, -1, -1], [-1,  4, -1, -1],
           [-1, -1, -1,  1], [-1, -1,  0,  2], [-1, -1,  1, -1]])
    """
    if cell_count(shape) > 0:
        shape = np.array(shape) - 2
        ids = node_index_with_halo(shape, halo_indices=out_of_bounds)

        diags = np.vstack((
            ids[2:, 2:].flat,
            ids[2:, :shape[1]].flat,
            ids[:shape[0], :shape[1]].flat,
            ids[:shape[0], 2:].flat,)).T
        if contiguous:
            return diags.copy()
        else:
            return diags
    else:
        return np.array([], dtype=np.int)


def diagonal_array_slow(shape, out_of_bounds=BAD_INDEX_VALUE):
    (nrows, ncols) = shape
    ncells = shape[0] * shape[1]
    diagonal_cells = - np.ones([ncells, 4], dtype=np.int)
    for r in xrange( 1, nrows-1 ):
        for c in xrange( 1, ncols-1 ):
            cell_id = r * ncols + c
            diagonal_cells[cell_id,2] = cell_id - ncols - 1 # bottom left
            diagonal_cells[cell_id,0] = cell_id + ncols + 1 # top right
            diagonal_cells[cell_id,3] = cell_id - ncols + 1 # bottom right
            diagonal_cells[cell_id,1] = cell_id + ncols - 1 # top left
    return diagonal_cells


def has_boundary_neighbor(neighbors, diagonals,
                          out_of_bounds=BAD_INDEX_VALUE):
    return (out_of_bounds in neighbors |
            out_of_bounds in diagonals)


def has_boundary_neighbor_slow(neighbors, diagonals, out_of_bounds=BAD_INDEX_VALUE):
        #nbr_nodes=self.get_neighbor_list(id)
        #diag_nbrs=self.get_diagonal_list(id)
        
        i=0
        while i < 4 and neighbors[i] != out_of_bounds:
            i += 1
        
        if i < 4:
            return True
        else:
            r = 0
            while r < 4 and diagonals[r] != out_of_bounds:
                r += 1
        
        if r < 4 :
            return True
        else:
            return False


def reshape_array(shape, u, flip_vertically=False, copy=False):
    """
    >>> x = np.arange(12.)
    >>> y = reshape_array((3, 4), x)
    >>> y.shape
    (3, 4)
    >>> y
    array([[  0.,   1.,   2.,   3.],
           [  4.,   5.,   6.,   7.],
           [  8.,   9.,  10.,  11.]])
    >>> y.flags['C_CONTIGUOUS']
    True
    >>> x[0] = -1
    >>> y[0, 0]
    -1.0

    >>> x = np.arange(12.)
    >>> y = reshape_array((3, 4), x, flip_vertically=True)
    >>> y
    array([[  8.,   9.,  10.,  11.],
           [  4.,   5.,   6.,   7.],
           [  0.,   1.,   2.,   3.]])
    >>> y.flags['C_CONTIGUOUS']
    False
    >>> x[0] = -1
    >>> y[-1, 0]
    -1.0
    """
    reshaped_u = u.view()

    try:
        reshaped_u.shape = shape
    except ValueError:
        raise

    if flip_vertically:
        flipped_u = reshaped_u[::-1, :]
        if copy:
            return flipped_u.copy()
        else:
            return flipped_u
    else:
        if copy:
            return reshaped_u.copy()
        else:
            return reshaped_u

def nodes_around_points_on_unit_grid(shape, coords, mode='raise'):
    """
    Returns the nodes around a point on a structured grid with unit spacing
    and zero origin.

    >>> nodes_around_points_on_unit_grid((3, 3), (.1, .1))
    array([0, 3, 4, 1])

    >>> nodes_around_points_on_unit_grid((3, 3), (1., 1.))
    array([4, 7, 8, 5])
    """
    if isinstance(coords[0], np.ndarray):
        (rows, cols) = (coords[0].astype(np.int), coords[1].astype(np.int))
    else:
        (rows, cols) = (int(coords[0]), int(coords[1]))

    try:
        return np.ravel_multi_index(((rows, rows + 1, rows + 1, rows),
                                     (cols, cols, cols + 1, cols + 1)),
                                    shape, mode=mode).T
    except ValueError as e:
        raise

def nodes_around_points(shape, coords, spacing=(1., 1.),
                        origin=(0., 0.)):
    """
    Returns the nodes around a point on a structured grid with row and column
    *spacing*, and *origin*.

    >>> x = np.array([.9, 1.])
    >>> y = np.array([.1, 1.])
    >>> nodes_around_points((3, 3), (y, x))
    array([[0, 3, 4, 1],
           [4, 7, 8, 5]])

    >>> nodes_around_points((3, 3), (2., 1.))
    Traceback (most recent call last):
        ...
    ValueError: invalid entry in coordinates array
    """
    try:
        return nodes_around_points_on_unit_grid(
            shape,
            ((coords[0] - origin[0]) / spacing[0],
             (coords[1] - origin[1]) / spacing[1]))
    except ValueError as e:
        raise

def nodes_around_point(shape, coords, spacing=(1., 1.)):
    node_id = int(coords[0] // spacing[0] * shape[1] + coords[1] // spacing[1])
    if node_id + shape[1] + 1 >= shape[0] * shape[1] or node_id < 0:
        raise ValueError('invalid entry in coordinates array')

    return np.array([node_id, node_id + shape[1], node_id + shape[1] + 1,
                     node_id + 1])



#!/usr/bin/env python

import numpy as np

from landlab import RasterModelGrid, Component
from landlab.components.flexure.funcs import subside_point_loads


_VALID_METHODS = set(['airy', 'flexure'])


def assert_method_is_valid(method):
    if method not in _VALID_METHODS:
        raise ValueError('%s: Invalid method name' % method)


class FlexureComponent(Component):
    """
    Landlab component that implements a 1 and 2D lithospheric flexure
    model.

    >>> grid = RasterModelGrid(5, 4, 1.e4)
    >>> flex = FlexureComponent(grid)
    >>> flex.name
    'Flexure'
    >>> sorted(flex.input_var_names)
    ['lithosphere__elevation', 'lithosphere__overlying_pressure', 'planet_surface_sediment__deposition_increment']
    >>> sorted(flex.output_var_names)
    ['lithosphere__elevation', 'lithosphere__elevation_increment']
    >>> for var in sorted(flex.units): flex.units[var]
    'm'
    'm'
    'Pa'
    'm'

    >>> flex.grid.number_of_node_rows
    5
    >>> flex.grid.number_of_node_columns
    4
    >>> flex.grid is grid
    True

    >>> np.all(grid.at_node['lithosphere__elevation'] == 0.)
    True

    >>> np.all(grid.at_node['lithosphere__elevation'] == 0.)
    True
    >>> np.all(grid.at_node['lithosphere__overlying_pressure'] == 0.)
    True
    >>> flex.update()
    >>> np.all(grid.at_node['lithosphere__elevation_increment'] == 0.)
    True

    >>> load = grid.at_node['lithosphere__overlying_pressure']
    >>> load[4] = 1e9
    >>> dz = grid.at_node['lithosphere__elevation_increment']
    >>> np.all(dz == 0.)
    True

    >>> flex.update()
    >>> np.all(grid.at_node['lithosphere__elevation_increment'] == 0.)
    False
    """
    _name = 'Flexure'

    _input_var_names = set([
        'lithosphere__overlying_pressure',
        'lithosphere__elevation',
        'planet_surface_sediment__deposition_increment',
    ])
    _output_var_names = set([
        'lithosphere__elevation_increment',
        'lithosphere__elevation',
    ])

    _var_units = {
        'lithosphere__overlying_pressure': 'Pa',
        'lithosphere__elevation': 'm',
        'lithosphere__elevation_increment': 'm',
        'planet_surface_sediment__deposition_increment': 'm',
    }

    def __init__(self, grid, **kwds):
        self._eet = kwds.pop('eet', 65000.)
        self._youngs = kwds.pop('youngs', 7e10)
        self._method = kwds.pop('method', 'airy')

        assert_method_is_valid(self._method)

        super(FlexureComponent, self).__init__(grid, **kwds)

        for name in self._input_var_names:
            if not name in self.grid.at_node:
                self.grid.add_zeros('node', name, units=self._var_units[name])

        for name in self._output_var_names:
            if not name in self.grid.at_node:
                self.grid.add_zeros('node', name, units=self._var_units[name])

        self._last_load = self.grid.field_values('node', 'lithosphere__overlying_pressure').copy()

        self._nodal_values = self.grid['node']

    def update(self, n_procs=1):
        elevation = self._nodal_values['lithosphere__elevation']
        load = self._nodal_values['lithosphere__overlying_pressure']
        deflection = self._nodal_values['lithosphere__elevation_increment']
        deposition = self._nodal_values['planet_surface_sediment__deposition_increment']

        new_load = ((load - self._last_load) +
                    (deposition * 2650. * 9.81).flat)

        self._last_load = load.copy()

        deflection.fill(0.)

        if self._method == 'airy':
            deflection[:] = new_load / (3300. * 9.81)
        else:
            self.subside_loads(new_load, self.coords, deflection=deflection,
                               n_procs=n_procs)

        elevation -= deflection

    def subside_loads(self, loads, locs, deflection=None, n_procs=1):
        if deflection is None:
            deflection = np.empty(self.shape, dtype=np.float)

        subside_point_loads(loads, locs, self.coords, self._eet, self._youngs,
                            deflection=deflection, n_procs=n_procs)

        return deflection

    def subside_load(self, load, loc, deflection=None):
        subside_point_load(
            load, loc, self.coords, self._eet, self._youngs,
            deflection=self._nodal_values['lithosphere__elevation_increment'])

        return deflection


if __name__ == "__main__":
    import doctest
    doctest.testmod()



import numpy as np
from numpy.testing import assert_array_equal
from nose.tools import (assert_is, assert_equal, assert_raises, raises,
                        assert_true, assert_false)

from landlab import RasterModelGrid


class TestRasterModelGridConnectingFaces():
    def setup(self):
        self.rmg = RasterModelGrid(4, 5)

    def test_horizontally_adjacent_cells(self):
        assert_array_equal(self.rmg.get_face_connecting_cell_pair(0, 1),
                           np.array([10]))

    def test_vertically_adjacent_cells(self):
        assert_array_equal(self.rmg.get_face_connecting_cell_pair(0, 3),
                           np.array([3]))

    def test_diagonally_adjacent_cells(self):
        assert_array_equal(self.rmg.get_face_connecting_cell_pair(1, 5),
                           np.array([]))

    def test_non_adjacent_cells(self):
        assert_array_equal(self.rmg.get_face_connecting_cell_pair(0, 2),
                           np.array([]))


class TestRasterModelGridCellFaces():
    def setup(self):
        self.rmg = RasterModelGrid(4, 5)

    def test_id_as_int(self):
        assert_array_equal(self.rmg.cell_faces(0), np.array([0, 9, 3, 10]))

    def test_id_as_array(self):
        assert_array_equal(self.rmg.cell_faces(np.array([0, 1])),
                           np.array([[0, 9, 3, 10], [1, 10, 4, 11]]))



#! /usr/env/python

"""
fracture_grid: creates and returns a 2D grid with randomly generated fractures.
The grid contains the value 1 where fractures (one cell wide) exist, and
0 elsewhere. The idea is to use this for simulations based on weathering and
erosion of, and/or flow within, fracture networks.

The entry point is the function:
    
    make_frac_grid(frac_spacing, numrows=50, numcols=50, model_grid=None)
    
If called with a Landlab RasterModelGrid, the function returns the fracture
grid as a node array. Otherwise, it returns a numrows x numcols Numpy array.

Potential improvements:
    - Add doctests
    - Fractures could be defined by links rather than nodes (i.e., return a
        link array with a code indicating whether the link crosses a fracture
        or not)
    - Fractures could have a finite length rather than extending all the way
        across the grid
        
Created: September 2013 by Greg Tucker
Last significant modification: xxx
"""

from numpy import *
from numpy.random import *
import random
import pylab as plt


def calculate_fracture_starting_position(numrows, numcols):
    
    if random.randint(0, 1)==0:
        x = 0
        y = random.randint(0, numrows-1)
    else:
        x = random.randint(0, numcols-1)        
        y = 0
    return x, y
    
    
def calculate_fracture_orientation(x, y):
    
    ang = (pi/2)*rand()
    if y==0:
        ang += pi/4
    else:
        ang -= pi/4

    return ang

    
def calculate_fracture_step_sizes(startx, starty, ang):
    
    if startx==0:  # frac starts on left side
        dx = 1
        dy = tan(ang)
    else:  # frac starts on bottom side
        dy = 1
        dx = -tan(ang-pi/2)
        
    return dx, dy
    
    
def trace_fracture_through_grid(m, x0, y0, dx, dy):
    
    x = x0
    y = y0
    
    while round(x)<size(m, 1) and round(y)<size(m, 0) \
            and round(x)>=0 and round(y)>=0:
        #print x, y, size(m, 1), size(m, 0)
        m[round(y),round(x)] = 1
        x += dx
        y += dy


def make_frac_grid(frac_spacing, numrows=50, numcols=50, model_grid=None):
    
    # Make an initial grid of all zeros. If user specified a model grid,
    # use that. Otherwise, use the given dimensions.
    if model_grid is not None:
        numrows = model_grid.number_of_node_rows
        numcols = model_grid.number_of_node_columns
    m = zeros((numrows,numcols))
    
    # Add fractures to grid
    nfracs = (numrows+numcols)/frac_spacing
    for i in range(nfracs):
        
        x, y = calculate_fracture_starting_position(numrows, numcols)
        ang = calculate_fracture_orientation(x, y)
        dx, dy = calculate_fracture_step_sizes(x, y, ang)
    
        #print 'startx=',x,'starty=',y,'ang=',180*ang/pi,'dx=',dx,'dy=',dy
    
        trace_fracture_through_grid(m, x, y, dx, dy)
    
    # If we have a model_grid, flatten the frac grid so it's equivalent to
    # a node array.
    if model_grid is not None:
        #print 'FROG!'
        m.shape = (m.shape[0]*m.shape[1])

    return m
            
    
def main():
    
    # User-defined parameters
    N = 100
    frac_spacing = 8
    
    frac_grid = make_frac_grid(frac_spacing, N, N)
    
    plt.figure()
    plt.imshow(frac_grid)
    plt.show()
        
    # Version with model grid
    from landlab import RasterModelGrid
    rmg = RasterModelGrid(40, 50, 1.0)
    frac_grid = make_frac_grid(frac_spacing, model_grid=rmg)
    fg_raster = rmg.node_vector_to_raster(frac_grid)
    
    plt.figure()
    plt.imshow(fg_raster)
    plt.show()
        
    
    
if __name__== "__main__":
    main()
    



########## soil_moisture.py ###############
##
##  This component calculates and updates soil moisture after each storm.
##  Soil moisture is represented as a single bucket. Rainfall depth fills the bucket
##  and the soil moisture decays as a result of leakage and ET following the analytical solution of
##  Laio et al., (2001) 
##  Input file is named soilmoisture_input.txt and is temporarily placed under landlab.components.
##  All input values can be altered using set_*** functions
##  All output arrays can be retrieved using get_*** functions
##
##  Storms are considered to be instantaneous events.
##  Storm duration, depth and interstorm duration are obtained from RainfallDriver.py
##  Storm depth is obtained in mm and storm duration and interstorm duration are obtained in hours
##
##  Sai Nudurupati and E.I. 08/07/2013.
###########################################

import os
from numpy import *
from math import *
import landlab
from landlab.components.uniform_precip.generate_uniform_precip import PrecipitationDistribution
from landlab.model_parameter_dictionary import ModelParameterDictionary
from matplotlib.pyplot import *


_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'soilmoisture_input_validation_high.txt')

class SoilMoisture():

    def __init__( self ):
        """ vegcover - vegetation cover fraction """
        self._vegcover = 0.0
        """ interception_cap - Maximum interception capacity """
        self._interception_cap = 0.0
        """ zr - root depth """
        self._zr = 0.0
        """ runon - runon from upstream """
        self._runon = 0.0
        """ PET - potential evapotranspiration """
        self._PET = 0.0
        """ fbare - scaling coefficient for bare soil Potential Evapotranspiration """
        self._fbare = 0.0
        """ Ib - Bare soil infiltration capacity [mm/h] """
        self._soil_Ib = 0.0
        """ Iv - Vegetated surface infiltration capacity [mm/h] """
        self._soil_Iv = 0.0
        """ Ew - Residual Evaporation after wilting """
        self._soil_Ew = 0.0
        """ pc - porosity """
        self._soil_pc = 0.0
        """ fc - field capacity """
        self._soil_fc = 0.0
        """ sc - stomata closure moisture fraction """
        self._soil_sc = 0.0
        """ wp - wilting point """ 
        self._soil_wp = 0.0
        """ hgw - Hygroscopic water """
        self._soil_hgw = 0.0
        """ beta - Deep percolation rate constant """
        self._soil_beta = 0.0
     
        
        """ initialize """
        P = 0
        Tb = 0
        Tr = 0
        ETA = 0.0
        Water_Stress = 0.0
        
        """ sini - soil moisture with storm at the end of time intervals """
        sini = 0.0
        """ s - final soil moisture """
        s = 0.0
        """ Dd - Drainage from rootzone """
        Dd = 0.0
        """ Runoff - runoff after storm """
        Runoff = 0.0
        
        """ Iterations - Number of iterations/storms """
        self._iterate_storm = 0
        """ so - Soil moisture of the ground before storm """
        so = 0.0

        """ Initialize output variables  """
        
        self._Peff = 0.0
        self._ETA = 0.0
        self._S = 0.0
        self._D = 0.0
        self._Ro = 0.0
        self._time = 0.0
        self._WS = 0.0    # water stress
        self._LAIlive = 0.0
        self._fr = 0.0
       
    def initialize( self ):

        MPD = ModelParameterDictionary()
        MPD.read_from_file(_DEFAULT_INPUT_FILE)

        self._iterate_storm = MPD.read_int( 'N_STORMS' )
        self._vegcover = MPD.read_float( 'VEG_COV' )
        self._interception_cap = MPD.read_float( 'INTERCEPT_CAP' )
        self._zr = MPD.read_float( 'ZR' )
        self._runon = MPD.read_float( 'RUNON' )
        self._PET = MPD.read_float( 'PET' )
        self._fbare = MPD.read_float( 'F_BARE' )
        self._so = MPD.read_float( 'so' )        
        self._si = self._so
        self._soil_Ib = MPD.read_float( 'I_B' )
        self._soil_Iv = MPD.read_float( 'I_V' )
        self._soil_Ew = MPD.read_float( 'EW' )
        self._soil_pc = MPD.read_float( 'PC' )
        self._soil_fc = MPD.read_float( 'FC' )
        self._soil_sc = MPD.read_float( 'SC' )
        self._soil_wp = MPD.read_float( 'WP' )
        self._soil_hgw = MPD.read_float( 'HGW' )
        self._soil_beta = MPD.read_float( 'BETA' )

        
        

    def update( self, P, Tb, Tr, fveg, PET ):

        V = self._vegcover   # total veg cover used for interception
        fbare = self._fbare
        ZR = self._zr
        #PET = self._PET
        pc = self._soil_pc
        fc = self._soil_fc
        sc = self._soil_sc
        wp = self._soil_wp
        hgw = self._soil_hgw
        beta = self._soil_beta
        self._fr = self._LAIlive/1.44

        
        Inf_cap = self._soil_Ib*(1-V) + self._soil_Iv*V
        Int_cap = min(V*self._interception_cap, P)
        Peff = max(P-Int_cap, 0.0)
        mu = (Inf_cap/1000.0)/(pc*ZR*(exp(beta*(1-fc))-1))
        Ep = max((PET*self._fr+fbare*PET*(1-self._fr)) - Int_cap, 0.001)
        nu = ((Ep/24.0)/1000.0)/(pc*ZR)
        nuw = ((Ep*0.1/24)/1000.0)/(pc*ZR)
        sini = self._so + (Peff/(pc*ZR*1000.0))+self._runon
        

        if sini>1:
           runoff = (sini-1)*pc*ZR*1000
           sini = 1
        else:
           runoff = 0

        if sini>=fc:
            tfc = (1.0/(beta*(mu-nu)))*(beta*(fc-sini)+log((nu-mu+mu*exp(beta*(sini-fc)))/nu))
            tsc = ((fc-sc)/nu)+tfc
            twp = ((sc-wp)/(nu-nuw))*log(nu/nuw)+tsc

            if Tb<tfc:
                s = abs(sini-(1/beta)*log(((nu-mu+mu*exp(beta*(sini-fc)))*exp(beta*(nu-mu)*Tb)-mu*exp(beta*(sini-fc)))/(nu-mu)))
                Dd = ((pc*ZR*1000)*(sini-s))-(Tb*(Ep/24))
                ETA = (Tb*(Ep/24))

            elif Tb>=tfc and Tb<tsc:
                s = fc-(nu*(Tb-tfc))
                Dd = ((pc*ZR*1000)*(sini-fc))-((tfc)*(Ep/24))
                ETA = (Tb*(Ep/24))

            elif Tb>=tsc and Tb<twp:
                s = wp+(sc-wp)*((nu/(nu-nuw))*exp((-1)*((nu-nuw)/(sc-wp))*(Tb-tsc))-(nuw/(nu-nuw)))
                Dd = ((pc*ZR*1000)*(sini-fc))-(tfc*Ep/24)
                ETA = (1000*ZR*pc*(sini-s))-Dd

            else:
                s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*max(Tb-twp,0))
                Dd = ((pc*ZR*1000)*(sini-fc))-(tfc*Ep/24)
                ETA = (1000*ZR*pc*(sini-s))-Dd

        elif sini<fc and sini>=sc:
            tfc = 0
            tsc = (sini-sc)/nu
            twp = ((sc-wp)/(nu-nuw))*log(nu/nuw)+tsc

            if Tb<tsc:
                s = sini - nu*Tb
                Dd = 0
                ETA = 1000*ZR*pc*(sini-s)

            elif Tb>=tsc and Tb<twp:
                s = wp+(sc-wp)*((nu/(nu-nuw))*exp((-1)*((nu-nuw)/(sc-wp))*(Tb-tsc))-(nuw/(nu-nuw)))
                Dd = 0
                ETA = (1000*ZR*pc*(sini-s))

            else:
                s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*(Tb-twp))
                Dd = 0
                ETA = (1000*ZR*pc*(sini-s))
                
        elif sini<sc and sini>=wp:
            tfc = 0
            tsc = 0
            twp = ((sc-wp)/(nu-nuw))*log(1+(nu-nuw)*(sini-wp)/(nuw*(sc-wp)))

            if Tb<twp:
                s = wp+((sc-wp)/(nu-nuw))*((exp((-1)*((nu-nuw)/(sc-wp))*Tb))*(nuw+((nu-nuw)/(sc-wp))*(sini-wp))-nuw)
                Dd = 0
                ETA = (1000*ZR*pc*(sini-s))

            else:
                s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*(Tb-twp))
                Dd = 0
                ETA = (1000*ZR*pc*(sini-s))

        else:
            tfc = 0
            tsc = 0
            twp = 0

            s = hgw+(sini-hgw)*exp((-1)*(nuw/(wp-hgw))*Tb)
            Dd = 0
            ETA = (1000*ZR*pc*(sini-s))

        Water_Stress = min(max(pow(((sc - ((s+sini)/2)) / (sc - wp)),4),0.0),1.0) 
        
        #print 'soil moisture: ', s
    
        """ varaibles for get/set functions """
        self._Peff = Peff
        self._ETA = ETA
        self._S = s
        self._D = Dd
        self._Ro = runoff
        self._time = self._time + (Tb + Tr)/(24*365.4)
        self._WS = Water_Stress
        self._so = s
        self._tfc = tfc
        self._tsc = tsc
        self._twp = twp
        self._mu = mu
        self._nu = nu
        self._nuw = nuw   

        """ Return Effective precipittion mm """
    def get_Peff( self ):
        return self._Peff
      
    def get_time( self ):
        return self._time
 
        """ Return Interstorm ET mm """
    def get_ET( self ):
        return self._ETA

        """ Return Soil Moisture  """
    def get_S( self ):
        return self._S

        """ Return Drainage from Root Zone mm """
    def get_D( self ):
        return self._D

        """ Return Runoff mm """
    def get_Ro( self ):
        return self._Ro
    
        """ Return Water Stress """
    def get_WaterStress( self ):
        return self._WS
        
    def get_tfc( self ):
        return self._tfc
        
    def get_tsc( self ):
        return self._tsc
        
    def get_twp( self ):
        return self._twp
        
    def get_mu( self ):
        return self._mu

    def get_nu( self ):
        return self._nu

    def get_nuw( self ):
        return self._nuw

        """ Set Number of Storms """
    def set_strms( self, strms ):
        self._iterate_storm = strms
        
    def set_LAIlive( self, LAI ):
        self._LAIlive = LAI
        
        
        """ Set Potential Evapotranspiration Rate """
    def set_PET( self, PET ):
        self._PET = PET
        
    def set_vegcover( self, VegCovR ):
        self._vegcover = VegCovR


    def plott( self, P_, S_, ETA_, D_, Ro_, Time_ ):

        figure(1)
        plot( Time_, P_ )
        ylabel( 'Storm Depth [mm]' )
        title( 'Storm generation' )
        xlabel( 'Time:Year' )
        savefig('figure1')

        figure(2)
        plot( Time_, S_ )
        ylabel( 'Degree of saturation: theta/porosity' )
        title( 'Soil Moisture Dynamics' )
        xlabel( 'Time:Year' )
        savefig('figure2')

        figure(3)
        plot( Time_, ETA_, '--+', label = 'Interstorm-ET' )
        hold(True)
        plot( Time_, D_, '-', label = 'Root-zone Drainge' )
        hold(True)
        plot( Time_, Ro_, '*', label = 'Runoff' )
        hold(False)
        xlabel( 'Time:Year' )
        ylabel( 'Depth [mm]' )
        legend()
        title( 'Leakage/Losses' )
        savefig('figure3')

        show()


import numpy as np
from numpy.testing import assert_array_equal
from nose import with_setup
from nose.tools import assert_is, assert_equal, raises

from landlab.grid import raster_funcs as rfuncs


def setup_grid():
    from landlab import RasterModelGrid
    globals().update({
        'rmg': RasterModelGrid(4, 5),
        'values_at_nodes':  np.arange(20.),
    })


@with_setup(setup_grid)
def test_corner_with_scalar_cell_id():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, np.array([0]), 0)
    assert_array_equal(node_ids, np.array([2]))


@with_setup(setup_grid)
def test_face_with_scalar_cell_id():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, np.array([0]), 0)
    assert_array_equal(node_ids, np.array([1]))


@with_setup(setup_grid)
def test_corner_with_iterable_cell_id():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, np.array([0]), (5, ))
    assert_array_equal(node_ids, np.array([9]))


@with_setup(setup_grid)
def test_face_with_iterable_cell_id():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, np.array([0]), (5, ))
    assert_array_equal(node_ids, np.array([8]))


@with_setup(setup_grid)
def test_corner_with_no_cell_id():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, np.array([0]))
    assert_array_equal(node_ids, np.array([2, 3, 4, 7, 8, 9]))


@with_setup(setup_grid)
def test_face_with_no_cell_id():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, np.array([0]))
    assert_array_equal(node_ids, np.array([1, 2, 3, 6, 7, 8]))


@with_setup(setup_grid)
def test_corner_multiple_corners():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, np.array([0, 1]), (4, ))
    assert_array_equal(node_ids, np.array([8, 6]))


@with_setup(setup_grid)
def test_face_multiple_faces():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, np.array([0, 1]), (4, ))
    assert_array_equal(node_ids, np.array([7, 11]))


@with_setup(setup_grid)
def test_corner_multiple_corners_and_cells():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, np.array([0, 1]), (4, 5))
    assert_array_equal(node_ids, np.array([8, 7]))


@with_setup(setup_grid)
def test_face_multiple_corners_and_cells():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, np.array([0, 1]), (4, 5))
    assert_array_equal(node_ids, np.array([7, 12]))


@with_setup(setup_grid)
def test_corner_type_tuple():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, (0, 1), (4, 5))
    assert_array_equal(node_ids, np.array([8, 7]))


@with_setup(setup_grid)
def test_face_type_tuple():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, (0, 1), (4, 5))
    assert_array_equal(node_ids, np.array([7, 12]))


@with_setup(setup_grid)
def test_corner_type_list():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, [0, 1], (4, 5))
    assert_array_equal(node_ids, np.array([8, 7]))


@with_setup(setup_grid)
def test_face_type_list():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, [0, 1], (4, 5))
    assert_array_equal(node_ids, np.array([7, 12]))


@with_setup(setup_grid)
def test_corner_type_scalar():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, 2, (4, ))
    assert_array_equal(node_ids, np.array([16]))

    node_ids = rfuncs.node_id_of_cell_corner(rmg, 2, (4, 5))
    assert_array_equal(node_ids, np.array([16, 17]))

    node_ids = rfuncs.node_id_of_cell_corner(rmg, 1)
    assert_array_equal(node_ids, np.array([0, 1, 2, 5, 6, 7]))


@with_setup(setup_grid)
def test_face_type_scalar():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, 2, (4, ))
    assert_array_equal(node_ids, np.array([17]))

    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, 2, (4, 5))
    assert_array_equal(node_ids, np.array([17, 18]))

    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, 1)
    assert_array_equal(node_ids, np.array([5, 6, 7, 10, 11, 12]))


@raises(ValueError)
@with_setup(setup_grid)
def test_corner_length_mismatch():
    node_ids = rfuncs.node_id_of_cell_corner(rmg, (2, 3), (2, 4, 5))


@raises(ValueError)
@with_setup(setup_grid)
def test_face_length_mismatch():
    node_ids = rfuncs.node_id_of_cell_neighbor(rmg, (2, 3), (2, 4, 5))



#! /usr/bin/env python

import unittest
import numpy as np

from landlab.testing import NumpyArrayTestingMixIn

import landlab.utils.structured_grid as sgrid
from landlab.utils.structured_grid import BAD_INDEX_VALUE


class TestGetNodeCoords(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_node_x_2d(self):
        (x, _) = sgrid.node_coords((3,2))

        self.assertArrayEqual(x, np.array([0., 1.,
                                        0., 1.,
                                        0., 1.,]))

    def test_node_x_2d_with_spacing(self):
        (x, _) = sgrid.node_coords((3,2), (2., 10.))

        self.assertArrayEqual(x, np.array([0., 10.,
                                        0., 10.,
                                        0., 10.,]))

    def test_node_x_2d_with_origin(self):
        (x, _) = sgrid.node_coords((3,2), (2., 10.), (-1., 1.))

        self.assertArrayEqual(x, np.array([1., 11.,
                                        1., 11.,
                                        1., 11.,]))

    def test_node_y_2d(self):
        (_, y) = sgrid.node_coords((3,2))

        self.assertArrayEqual(y, np.array([0., 0.,
                                        1., 1.,
                                        2., 2.,]))

    def test_node_y_2d_with_spacing(self):
        (_, y) = sgrid.node_coords((3,2), (2., 10.))

        self.assertArrayEqual(y, np.array([0., 0.,
                                        2., 2.,
                                        4., 4.,]))

    def test_node_y_2d_with_origin(self):
        (_, y) = sgrid.node_coords((3,2), (2., 10.), (-1., 1.))

        self.assertArrayEqual(y, np.array([-1., -1.,
                                         1.,  1.,
                                         3.,  3.,]))

    def test_round_off_error(self):
        (x, y) = sgrid.node_coords((135, 127),
                                   (5.4563957090392, 5.4563957090392),
                                   (0., 0.))

        self.assertTupleEqual(x.shape, (135 * 127, ))
        self.assertTupleEqual(y.shape, (135 * 127, ))


class TestGetCellNode(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_2d_shape_2_by_3(self):
        cell_nodes = sgrid.node_index_at_cells((2, 3))

        self.assertArrayEqual(cell_nodes, np.array([]))

    def test_2d_shape_3_by_3(self):
        cell_nodes = sgrid.node_index_at_cells((3, 3))

        self.assertArrayEqual(cell_nodes, np.array([4]))

    def test_shape_4_by_5(self):
        cell_nodes = sgrid.node_index_at_cells((4, 5))

        self.assertArrayEqual(cell_nodes, np.array([ 6,  7,  8,
                                                 11, 12, 13]))


class TestGetNodeLinks(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_2d_3_by_2_from_links(self):
        (from_indices, _) = sgrid.node_index_at_link_ends((3, 2))

        self.assertArrayEqual(from_indices,
                           np.array([0, 1, 2, 3,
                                     0, 2, 4]))

    def test_2d_3_by_2_to_links(self):
        (_, to_indices) = sgrid.node_index_at_link_ends((3, 2))

        self.assertArrayEqual(to_indices,
                           np.array([2, 3, 4, 5,
                                     1, 3, 5]))

    def test_west_links(self):
        links = sgrid.west_links((3, 4))
        self.assertArrayEqual(links,
                           np.array([[-1,  8,  9, 10],
                                     [-1, 11, 12, 13],
                                     [-1, 14, 15, 16]]))

        links = sgrid.west_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, 0, 1, 2]]))

        links = sgrid.west_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1], [-1], [-1], [-1]]))

    def test_east_links(self):
        links = sgrid.east_links((3, 4))
        self.assertArrayEqual(links,
                           np.array([[ 8,  9, 10, -1],
                                     [11, 12, 13, -1],
                                     [14, 15, 16, -1]]))

        links = sgrid.east_links((1, 4))
        self.assertArrayEqual(links, np.array([[0, 1, 2, -1]]))

        links = sgrid.east_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]).T)

    def test_north_links(self):
        links = sgrid.north_links((3, 4))
        self.assertArrayEqual(links, np.array([[ 0,  1,  2,  3],
                                            [ 4,  5,  6,  7],
                                            [-1, -1, -1, -1]]))

        links = sgrid.north_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.north_links((4, 1))
        self.assertArrayEqual(links, np.array([[0, 1, 2, -1]]).T)

    def test_south_links(self):
        links = sgrid.south_links((3, 4))
        self.assertArrayEqual(links,
                           np.array([[-1, -1, -1, -1],
                                     [ 0,  1,  2,  3],
                                     [ 4,  5,  6,  7]]))

        links = sgrid.south_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.south_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, 0, 1, 2]]).T)

    def test_inlinks(self):
        links = sgrid.inlinks((3, 4))
        self.assertArrayEqual(
            np.array([[-1, -1, -1, -1,  0,  1,  2,  3,  4,  5,  6,  7],
                      [-1,  8,  9, 10, -1, 11, 12, 13, -1, 14, 15, 16]]),
            links)

    def test_outlinks(self):
        links = sgrid.outlinks((3, 4))
        self.assertArrayEqual(
            np.array([[ 0,  1,  2,  3,  4,  5,  6,  7, -1, -1, -1, -1],
                      [ 8,  9, 10, -1, 11, 12, 13, -1, 14, 15, 16, -1]]),
            links)

class TestNodeActiveCell(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_one_active_cell(self):
        active_cells = sgrid.active_cell_index_at_nodes((3, 3))

        self.assertArrayEqual(
            active_cells,
            np.array([BAD_INDEX_VALUE, BAD_INDEX_VALUE, BAD_INDEX_VALUE,
                      BAD_INDEX_VALUE,               0, BAD_INDEX_VALUE,
                      BAD_INDEX_VALUE, BAD_INDEX_VALUE, BAD_INDEX_VALUE])
        )

    def test_no_active_cells(self):
        active_cells = sgrid.active_cell_index_at_nodes((3, 2))

        self.assertArrayEqual(
            active_cells,
            np.array([BAD_INDEX_VALUE, BAD_INDEX_VALUE,
                      BAD_INDEX_VALUE, BAD_INDEX_VALUE,
                      BAD_INDEX_VALUE, BAD_INDEX_VALUE])
        )
                                     
                                     
class TestActiveCells(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_one_active_cell(self):
        active_cells = sgrid.active_cell_index((3, 3))

        self.assertArrayEqual(active_cells, np.array([0]))

    def test_no_active_cells(self):
        active_cells = sgrid.active_cell_index((3, 2))

        self.assertArrayEqual(active_cells,
            np.array([]))


class TestCellCount(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_one_cell(self):
        n_cells = sgrid.cell_count((3, 3))
        self.assertEqual(n_cells, 1)

    def test_no_cells(self):
        n_cells = sgrid.cell_count((2, 3))
        self.assertEqual(n_cells, 0)


class TestInteriorCellCount(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_one_cell(self):
        n_cells = sgrid.interior_cell_count((3, 3))
        self.assertEqual(n_cells, 1)

    def test_no_cells(self):
        n_cells = sgrid.interior_cell_count((2, 3))
        self.assertEqual(n_cells, 0)


class TestActiveCellCount(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_one_cell(self):
        n_cells = sgrid.active_cell_count((3, 3))
        self.assertEqual(n_cells, 1)

    def test_no_cells(self):
        n_cells = sgrid.active_cell_count((2, 3))
        self.assertEqual(n_cells, 0)


class TestInteriorNodes(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_4_by_5(self):
        interiors = sgrid.interior_nodes((4, 5))
        self.assertArrayEqual(interiors, np.array([6, 7, 8, 11, 12, 13]))

    def test_no_interiors(self):
        interiors = sgrid.interior_nodes((2, 3))
        self.assertArrayEqual(interiors, np.array([]))


class TestNodeStatus(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_4_by_5(self):
        status = sgrid.node_status((4, 5))
        self.assertEqual(status.dtype, np.int8)
        self.assertArrayEqual(status,
                           np.array([1, 1, 1, 1, 1,
                                     1, 0, 0, 0, 1,
                                     1, 0, 0, 0, 1,
                                     1, 1, 1, 1, 1, ]))

    def test_no_interiors(self):
        status = sgrid.node_status((2, 3))
        self.assertEqual(status.dtype, np.int8)
        self.assertArrayEqual(status,
                           np.array([1, 1, 1,
                                     1, 1, 1,]))


class TestActiveLinks(unittest.TestCase, NumpyArrayTestingMixIn):
    """
    *--27-->*--28-->*--29-->*--30-->*
    ^       ^       ^       ^       ^
    10      11      12      13      14
    |       |       |       |       |
    *--23-->*--24-->*--25-->*--26-->*
    ^       ^       ^       ^       ^
    5       6       7       8       9   
    |       |       |       |       |
    *--19-->*--20-->*--21-->*--22-->*
    ^       ^       ^       ^       ^
    0       1       2       3       4
    |       |       |       |       |
    *--15-->*--16-->*--17-->*--18-->*
    """
    def test_4_by_5(self):
        active_links = sgrid.active_links((4, 5))
        self.assertArrayEqual(active_links,
                           np.array([1, 2, 3, 6, 7, 8, 11, 12, 13,
                                     19, 20, 21, 22, 23, 24, 25, 26]))
        self.assertEqual(len(active_links), sgrid.active_link_count((4, 5)))

    def test_with_node_status(self):
        status = sgrid.node_status((4, 5))
        status[6] = sgrid.INACTIVE_BOUNDARY
        active_links = sgrid.active_links((4, 5), node_status_array=status)

        self.assertArrayEqual(active_links,
                           np.array([2, 3, 7, 8, 11, 12, 13,
                                     21, 22, 23, 24, 25, 26]))

    def test_with_link_nodes(self):
        link_nodes = sgrid.node_index_at_link_ends((4, 5))
        active_links = sgrid.active_links((4, 5), link_nodes=link_nodes)

        self.assertArrayEqual(active_links,
                           np.array([1, 2, 3, 6, 7, 8, 11, 12, 13,
                                     19, 20, 21, 22, 23, 24, 25, 26]))
        self.assertEqual(len(active_links), sgrid.active_link_count((4, 5)))

    def test_vertical_active_link_count(self):
        link_count = sgrid.vertical_active_link_count((3, 4))
        self.assertEqual(4, link_count)

        link_count = sgrid.vertical_active_link_count((3, 2))
        self.assertEqual(0, link_count)

        node_status = np.ones((4, 5), dtype=np.int)
        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(9, link_count)

        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        node_status[0, 1] = 0
        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(8, link_count)

        node_status[2, 1] = 0
        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(6, link_count)

        node_status[2, 2] = 0
        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(4, link_count)

        node_status[1, 1] = 0
        link_count = sgrid.vertical_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(4, link_count)

    def test_horizontal_active_link_count(self):
        link_count = sgrid.horizontal_active_link_count((3, 4))
        self.assertEqual(3, link_count)

        link_count = sgrid.horizontal_active_link_count((2, 3))
        self.assertEqual(0, link_count)

        node_status = np.ones((4, 5), dtype=np.int)
        link_count = sgrid.horizontal_active_link_count(
            (4, 5), node_status=node_status)
        self.assertEqual(8, link_count)

        link_count = sgrid.horizontal_active_link_count(
            (4, 5), node_status=node_status)
        node_status[0, 1] = 0
        link_count = sgrid.horizontal_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(8, link_count)

        node_status[2, 1] = 0
        link_count = sgrid.horizontal_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(6, link_count)

        node_status[2, 2] = 0
        link_count = sgrid.horizontal_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(5, link_count)

        node_status[1, 1] = 0
        link_count = sgrid.horizontal_active_link_count((4, 5),
                                                      node_status=node_status)
        self.assertEqual(3, link_count)

    def test_horizontal_active_link_ids(self):
        links = sgrid.horizontal_active_link_ids((3, 4))
        self.assertArrayEqual(links, np.array([[4, 5, 6]]))

        links = sgrid.horizontal_active_link_ids((1, 4))
        expected = np.array([], ndmin=2, dtype=np.int64)
        expected.shape = (0, 3)
        self.assertArrayEqual(expected, links)

        links = sgrid.horizontal_active_link_ids((4, 1))
        expected.shape = (2, 0)
        self.assertArrayEqual(expected, links)

        node_status = np.ones((4, 5), dtype=int)
        links = sgrid.horizontal_active_link_ids((4, 5),
                                                 node_status=node_status)
        self.assertArrayEqual(links, np.array([[ 9, 10, 11, 12],
                                            [13, 14, 15, 16]]))

        node_status = np.ones((4, 5), dtype=int)
        node_status[1, 1] = 0
        links = sgrid.horizontal_active_link_ids((4, 5),
                                                 node_status=node_status)
        self.assertArrayEqual(links, np.array([[-1, -1,  7,  8],
                                            [ 9, 10, 11, 12]]))

        node_status[2, 1] = 0
        links = sgrid.horizontal_active_link_ids((4, 5),
                                                 node_status=node_status)
        self.assertArrayEqual(links, np.array([[-1, -1, 6, 7],
                                            [-1, -1, 8, 9]]))

        node_status[0, 0] = 0
        links = sgrid.horizontal_active_link_ids((4, 5),
                                                 node_status=node_status)
        self.assertArrayEqual(links, np.array([[-1, -1, 6, 7],
                                            [-1, -1, 8, 9]]))

    def test_vertical_active_link_ids(self):
        links = sgrid.vertical_active_link_ids((3, 4))
        self.assertArrayEqual(links, np.array([[0, 1], [2, 3]]))

        links = sgrid.vertical_active_link_ids((1, 4))
        expected = np.array([], ndmin=2, dtype=np.int64)
        expected.shape = (0, 2)
        self.assertArrayEqual(expected, links)

        links = sgrid.vertical_active_link_ids((4, 1))
        expected.shape = (3, 0)
        self.assertArrayEqual(expected, links)

        node_status = np.ones((4, 5), dtype=int)
        links = sgrid.vertical_active_link_ids((4, 5), node_status=node_status)
        self.assertArrayEqual(links, np.array([[0, 1, 2], [3, 4, 5], [6, 7, 8]]))

        node_status = np.ones((4, 5), dtype=int)
        node_status[1, 1] = 0
        links = sgrid.vertical_active_link_ids((4, 5), node_status=node_status)
        self.assertArrayEqual(links,
                           np.array([[-1, 0, 1], [-1, 2, 3], [4, 5, 6]]))

        node_status[2, 1] = 0
        links = sgrid.vertical_active_link_ids((4, 5), node_status=node_status)
        self.assertArrayEqual(links,
                           np.array([[-1, 0, 1], [-1, 2, 3], [-1, 4, 5]]))

        node_status[0, 0] = 0
        links = sgrid.vertical_active_link_ids((4, 5), node_status=node_status)
        self.assertArrayEqual(links,
                           np.array([[-1, 0, 1], [-1, 2, 3], [-1, 4, 5]]))

    def test_west_links(self):
        links = sgrid.active_west_links((3, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1],
                                            [-1,  4,  5,  6],
                                            [-1, -1, -1, -1]]))

        links = sgrid.active_west_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.active_west_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]).T)

    def test_east_links(self):
        links = sgrid.active_east_links((3, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1],
                                            [ 4,  5,  6, -1],
                                            [-1, -1, -1, -1]]))

        links = sgrid.active_east_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.active_east_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]).T)

        links = sgrid.horizontal_active_link_ids((4, 5))
        self.assertArrayEqual(np.array([[9, 10, 11, 12],
                                     [13, 14, 15, 16]]),
                           links)

        links = sgrid.active_east_links((4, 5))
        self.assertArrayEqual(np.array([[-1, -1, -1, -1, -1],
                                     [ 9, 10, 11, 12, -1],
                                     [13, 14, 15, 16, -1],
                                     [-1, -1, -1, -1, -1]]),
                           links)

    def test_north_links(self):
        links = sgrid.active_north_links((3, 4))
        self.assertArrayEqual(links, np.array([[-1,  0,  1, -1],
                                            [-1,  2,  3, -1],
                                            [-1, -1, -1, -1]]))

        links = sgrid.active_north_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.active_north_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]).T)

    def test_south_links(self):
        links = sgrid.active_south_links((3, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1],
                                            [-1,  0,  1, -1],
                                            [-1,  2,  3, -1]]))

        links = sgrid.active_south_links((1, 4))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]))

        links = sgrid.active_south_links((4, 1))
        self.assertArrayEqual(links, np.array([[-1, -1, -1, -1]]).T)

    def test_inlinks(self):
        links = sgrid.active_inlinks((3, 4))
        self.assertArrayEqual(
            np.array([[-1, -1, -1, -1, -1,  0,  1, -1, -1,  2,  3, -1],
                      [-1, -1, -1, -1, -1,  4,  5,  6, -1, -1, -1, -1]]),
            links)

    def test_outlinks(self):
        links = sgrid.active_outlinks((3, 4))
        self.assertArrayEqual(
            np.array([[-1,  0,  1, -1, -1,  2,  3, -1, -1, -1, -1, -1],
                      [-1, -1, -1, -1,  4,  5,  6, -1, -1, -1, -1, -1]]),
            links)

    def test_outlinks_4x5(self):
        links = sgrid.active_outlinks((4, 5))

        self.assertArrayEqual(np.array([[-1,  0,  1,  2, -1,
                                      -1,  3,  4,  5, -1,
                                      -1,  6,  7,  8, -1,
                                      -1, -1, -1, -1, -1],
                                     [-1, -1, -1, -1, -1,
                                      9, 10, 11, 12, -1,
                                      13, 14, 15, 16, -1,
                                      -1, -1, -1, -1, -1]]),
                           links)

    def test_inlinks_4x5(self):
        links = sgrid.active_inlinks((4, 5))

        self.assertArrayEqual(np.array([[-1, -1, -1, -1, -1,
                                      -1,  0,  1,  2, -1,
                                      -1,  3,  4,  5, -1,
                                      -1,  6, 7,  8, -1],
                                     [-1, -1, -1, -1, -1,
                                      -1,  9, 10, 11, 12,
                                      -1, 13, 14, 15, 16,
                                      -1, -1, -1, -1, -1]]),
                           links)


class TestFaces(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_face_count(self):
        self.assertEqual(17, sgrid.face_count((4, 5)))
        self.assertEqual(4, sgrid.face_count((3, 3)))
        self.assertEqual(0, sgrid.face_count((2, 100)))
        self.assertEqual(0, sgrid.face_count((100, 2)))
        self.assertEqual(0, sgrid.face_count((100, 1)))

    def test_active_face_count(self):
        self.assertEqual(17, sgrid.active_face_count((4, 5)))
        self.assertEqual(4, sgrid.active_face_count((3, 3)))
        self.assertEqual(0, sgrid.active_face_count((2, 100)))
        self.assertEqual(0, sgrid.active_face_count((100, 2)))
        self.assertEqual(0, sgrid.active_face_count((100, 1)))

    def test_active_faces(self):
        active_faces = sgrid.active_face_index((4, 5))
        self.assertArrayEqual(np.array([ 0,  1,  2,
                                      3,  4,  5,
                                      6,  7,  8,
                                      9, 10, 11, 12,
                                     13, 14, 15, 16]),
                           active_faces)


class TestLinkFaces(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_4_by_5(self):
        link_faces = sgrid.face_index_at_links((4, 5))

        BAD = sgrid.BAD_INDEX_VALUE

        self.assertArrayEqual(link_faces, np.array([BAD, 0, 1, 2, BAD,
                                                 BAD, 3, 4, 5, BAD,
                                                 BAD, 6, 7, 8, BAD,
                                                 BAD, BAD, BAD, BAD,
                                                 9, 10, 11, 12,
                                                 13, 14, 15, 16,
                                                 BAD, BAD, BAD, BAD]))

    def test_with_active_links(self):
        active_links = sgrid.active_links((4, 5))
        active_links = active_links[:-1]
        link_faces = sgrid.face_index_at_links((4, 5), actives=active_links)

        BAD = sgrid.BAD_INDEX_VALUE

        self.assertArrayEqual(link_faces, np.array([BAD, 0, 1, 2, BAD,
                                                 BAD, 3, 4, 5, BAD,
                                                 BAD, 6, 7, 8, BAD,
                                                 BAD, BAD, BAD, BAD,
                                                 9, 10, 11, 12,
                                                 13, 14, 15, BAD,
                                                 BAD, BAD, BAD, BAD]))


class TestReshapeArray(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_default(self):
        x = np.arange(12.)
        y = sgrid.reshape_array((3, 4), x)

        self.assertEqual(y.shape, (3, 4))
        self.assertArrayEqual(x, y.flat)
        self.assertTrue(y.flags['C_CONTIGUOUS'])
        self.assertIs(y.base, x)

    def test_copy(self):
        x = np.arange(12.)
        y = sgrid.reshape_array((3, 4), x, copy=True)

        self.assertEqual(y.shape, (3, 4))
        self.assertArrayEqual(x, y.flat)
        self.assertTrue(y.flags['C_CONTIGUOUS'])
        self.assertIsNone(y.base)

        y[0][0] = 0.
        self.assertArrayEqual(x, np.array([ 0.,  1.,  2., 3.,
                                         4.,  5.,  6., 7.,
                                         8.,  9., 10., 11.]))

    def test_flip(self):
        x = np.arange(12.)
        y = sgrid.reshape_array((3, 4), x, flip_vertically=True)

        self.assertEqual(y.shape, (3, 4))
        self.assertArrayEqual(y, np.array([[ 8.,  9., 10., 11.],
                                        [ 4.,  5.,  6., 7.],
                                        [ 0.,  1.,  2., 3.]]))
        self.assertFalse(y.flags['C_CONTIGUOUS'])
        self.assertIsNotNone(y.base)

        y[0][0] = 0.
        self.assertArrayEqual(x, np.array([ 0.,  1.,  2., 3.,
                                         4.,  5.,  6., 7.,
                                         0.,  9., 10., 11.]))

    def test_flip_copy(self):
        x = np.arange(12.)
        y = sgrid.reshape_array((3, 4), x, flip_vertically=True, copy=True)

        self.assertEqual(y.shape, (3, 4))
        self.assertArrayEqual(y, np.array([[ 8.,  9., 10., 11.],
                                        [ 4.,  5.,  6., 7.],
                                        [ 0.,  1.,  2., 3.]]))
        self.assertTrue(y.flags['C_CONTIGUOUS'])
        self.assertIsNot(y.base, x)


class TestDiagonalArray(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_default(self):
        diags = sgrid.diagonal_node_array((2, 3), out_of_bounds=-1)
        self.assertArrayEqual(diags,
                           np.array([[ 4, -1, -1, -1],
                                     [ 5,  3, -1, -1],
                                     [-1,  4, -1, -1],
                                     [-1, -1, -1,  1],
                                     [-1, -1,  0,  2],
                                     [-1, -1,  1, -1]]))

        self.assertTrue(diags.base is None)
        self.assertTrue(diags.flags['C_CONTIGUOUS'])

    def test_non_contiguous(self):
        diags = sgrid.diagonal_node_array((2, 3), out_of_bounds=-1,
                                          contiguous=False)
        self.assertArrayEqual(diags,
                           np.array([[ 4, -1, -1, -1],
                                     [ 5,  3, -1, -1],
                                     [-1,  4, -1, -1],
                                     [-1, -1, -1,  1],
                                     [-1, -1,  0,  2],
                                     [-1, -1,  1, -1]]))

        self.assertTrue(isinstance(diags.base, np.ndarray))
        self.assertFalse(diags.flags['C_CONTIGUOUS'])

    def test_boundary_node_mask_no_actives(self):
        diags = sgrid.diagonal_node_array((2, 3), out_of_bounds=-1,
                                          boundary_node_mask=-2)
        self.assertArrayEqual(diags, - 2 * np.ones((6, 4)))

    def test_boundary_node_mask(self):
        diags = sgrid.diagonal_node_array((3, 3), out_of_bounds=-1,
                                          boundary_node_mask=-2)
        self.assertArrayEqual(diags, 
                           np.array([[-2, -2, -2, -2], [-2, -2, -2, -2], [-2, -2, -2, -2],
                                     [-2, -2, -2, -2], [ 8,  6,  0,  2], [-2, -2, -2, -2],
                                     [-2, -2, -2, -2], [-2, -2, -2, -2], [-2, -2, -2, -2]]))


class TestNeighborArray(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_default(self):
        neighbors = sgrid.neighbor_node_array((2, 3))

        BAD = sgrid.BAD_INDEX_VALUE

        self.assertArrayEqual(
            neighbors,
            np.array([[  1,   3, BAD, BAD],
                      [  2,   4,   0, BAD],
                      [BAD,   5,   1, BAD],
                      [  4, BAD, BAD,   0],
                      [  5, BAD,   3,   1],
                      [BAD, BAD,   4,   2]]))

        self.assertTrue(neighbors.flags['C_CONTIGUOUS'])
        self.assertTrue(neighbors.base is None)

    def test_set_out_of_bounds(self):
        neighbors = sgrid.neighbor_node_array((2, 3), out_of_bounds=-1)
        self.assertArrayEqual(neighbors,
                           np.array([[ 1,  3, -1, -1],
                                     [ 2,  4,  0, -1],
                                     [-1,  5,  1, -1],
                                     [ 4, -1, -1,  0],
                                     [ 5, -1,  3,  1],
                                     [-1, -1,  4,  2]]))

    def test_as_view(self):
        neighbors = sgrid.neighbor_node_array((2, 3), out_of_bounds=-1,
                                              contiguous=False)
        self.assertArrayEqual(neighbors,
                           np.array([[ 1,  3, -1, -1],
                                     [ 2,  4,  0, -1],
                                     [-1,  5,  1, -1],
                                     [ 4, -1, -1,  0],
                                     [ 5, -1,  3,  1],
                                     [-1, -1,  4,  2]]))

        self.assertFalse(neighbors.flags['C_CONTIGUOUS'])
        self.assertTrue(isinstance(neighbors.base, np.ndarray))

    def test_boundary_node_mask_no_actives(self):
        neighbors = sgrid.neighbor_node_array((2, 3), out_of_bounds=-1,
                                          boundary_node_mask=-2)
        self.assertArrayEqual(neighbors, - 2 * np.ones((6, 4)))

    def test_boundary_node_mask(self):
        neighbors = sgrid.neighbor_node_array((3, 3), out_of_bounds=-1,
                                              boundary_node_mask=-2)
        self.assertArrayEqual(neighbors, 
                           np.array([[-2, -2, -2, -2], [-2, -2, -2, -2], [-2, -2, -2, -2],
                                     [-2, -2, -2, -2], [ 5,  7,  3,  1], [-2, -2, -2, -2],
                                     [-2, -2, -2, -2], [-2, -2, -2, -2], [-2, -2, -2, -2]]))


class TestInlinkMatrix(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_no_inactive(self):
        inlinks = sgrid.setup_active_inlink_matrix((4, 5), return_count=False)
        self.assertArrayEqual(inlinks,
                           np.array([[-1, -1, -1, -1, -1,
                                      -1,  0,  1,  2, -1,
                                      -1,  3,  4,  5, -1,
                                      -1,  6,  7,  8, -1],
                                     [-1, -1, -1, -1, -1,
                                      -1,  9, 10, 11, 12,
                                      -1, 13, 14, 15, 16,
                                      -1, -1, -1, -1, -1]]))

    def test_inactive(self):
        status = np.ones((4, 5))
        status[1, 1] = 0
        inlinks = sgrid.setup_active_inlink_matrix((4, 5), return_count=False,
                                                   node_status=status)
        self.assertArrayEqual(inlinks,
                           np.array([[-1, -1, -1, -1, -1,
                                      -1, -1,  0,  1, -1,
                                      -1, -1,  2,  3, -1,
                                      -1,  4,  5,  6, -1],
                                     [-1, -1, -1, -1, -1,
                                      -1, -1, -1,  7,  8,
                                      -1,  9, 10, 11, 12,
                                      -1, -1, -1, -1, -1]]))

    def test_out_link_ids_at_nodes(self):
        links_ids = sgrid.outlink_index_at_node((4, 5))
        self.assertArrayEqual(
            np.array([[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                       10, 11, 12, 13, 14, -1, -1, -1, -1, -1],
                      [15, 16, 17, 18, -1, 19, 20, 21, 22, -1,
                       23, 24, 25, 26, -1, 27, 28, 29, 30, -1]]),
            links_ids)

    def test_in_link_ids_at_nodes(self):
        links_ids = sgrid.inlink_index_at_node((4, 5))
        self.assertArrayEqual(
            np.array([[-1, -1, -1, -1, -1,  0,  1,  2,  3,  4,
                        5,  6,  7,  8,  9, 10, 11, 12, 13, 14],
                      [-1, 15, 16, 17, 18, -1, 19, 20, 21, 22,
                       -1, 23, 24, 25, 26, -1, 27, 28, 29, 30]]),
            links_ids)


if __name__ == '__main__':
    unittest.main()



######### VegetationDriver.py ###########
##
## 04 Aug 2013 - EI & SN
## 
## This code is an example for implementation of 
## single_vegetation
##
#############################################

# vegetation driver 

from single_vegetation import *
from soil_moisture import *


""" Create a Soil Moisture Object """
SM = SoilMoisture()
""" Initialize the Soil Moisture """
SM.initialize()

""" Create a Vegetation Object """
VEG = SingleVegetation()
""" Initialize Vegetation """
VEG.initialize()


SM._length = SM._iterate_storm

""" Create an instance of Storm Class """
PD = PrecipitationDistribution()
""" Initialize Storm - Create first storm """
PD.initialize()

#######

""" Loop through the storms """
for i in range( 0, SM._iterate_storm ):
    if i != 0:
       """ Create a new storm """ 
       PD.update()
    """ Soil moisture dynamics """
    P = PD.storm_depth
    Tb = PD.interstorm_duration
    Tr = PD.storm_duration

    """ Update Soil Moisture """
    SM.update( P, Tb, Tr )

    ETT = SM.get_ET()  # Since we are not sending the object SM to VEG.update,
    # we need to send the information of  SM.get_ET() as another variable - here ETT
    
    VEG.update(P, Tb, Tr, ETT ) # Note that ETT (variable - denoting
    # data from SM.get_ET()) 



#! /usr/env/python
"""

Example of a simple diffusion model that uses the DiffusionComponent.

Created July 2013 GT
Last updated August 2013 GT

"""

from landlab.components.diffusion import DiffusionComponent
from landlab.grid import create_and_initialize_grid
from landlab import ModelParameterDictionary
import pylab
import numpy


def display_model(grid, elevation):
    
    # Convert z to a 2D array
    zr = grid.node_vector_to_raster(elevation, flip_vertically=True)
    
    # Create a shaded image
    pylab.close()  # clear any pre-existing plot
    im = pylab.imshow(zr, cmap=pylab.cm.RdBu)  # display a colored image
    
    # add contour lines with labels
    # add a color bar on the side
    if numpy.amax(zr)>numpy.amin(zr):
        cset = pylab.contour(zr)
        pylab.clabel(cset, inline=True, fmt='%1.1f', fontsize=10)   
        pylab.colorbar(im)
    
    # add a title
    pylab.title('Simulated topography with uplift and diffusion')

    # Display the plot
    pylab.show()
    
    
def main():
    
    # INITIALIZE
    
    # Name of parameter input file
    input_file_name = 'test_inputs_for_diffusion_model.txt'
    
    # Open input file and read run-control parameters
    mpd = ModelParameterDictionary(input_file_name)
    run_duration = mpd.get('RUN_DURATION', ptype=float)
    
    # Create and initialize a grid
    mg = create_and_initialize_grid(mpd)
    
    # Create and initialize a diffusion component
    dc = DiffusionComponent(mg)
    dc.initialize(mpd)
    
    # RUN
    
    # Run the diffusion component until it's time for the next output
    dc.run_until(run_duration)
    
    # FINALIZE
    
    # Display results to screen
    mg.imshow('node', 'landscape_surface__elevation')
    import pylab
    pylab.show()
    
    from landlab.io.netcdf import write_netcdf
    write_netcdf('diffusion_example.nc', mg)
    
 
if __name__ == "__main__":
    main()



########## fire_generator.py #######
##
## This component generates random numbers
## using the Weibull distribution (Weibull, 1951).
## No particular units must be used, but it was
## written with the fire recurrence units in 
## time (yrs).
## 
## Using the Weibull Distribution assumes two things:
## All elements in the study area have the same fire regime.
## Each element must have (on average) a constant fire regime
## during the time span of the study (???? Problematic?!??)
##
## As of Sept, 2013, fires are considered instantaneous events
## independent of other fire events in the time series.
##
## Written by Jordan Adams, 2013.

## MOODY: FIRE RECURRENCE: 20 - 50 YRS
## 
## Elevation: 2,061 meters
##
## MFI according to Veblen et al., (2000)
##
## Fire years of >= 2 trees scarred: 13.4 years
## Fire years of 10% of trees scarred: 23.4 years
## Point fire interval: 63.3 years.


import os
from random import weibullvariate
from scipy import special
from landlab import ModelParameterDictionary

_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__), 'fire.txt')

class FireGenerator:

    def __init__(self):
        '''
        All initial values are set to zero until initalized
        using the initialize() method which reads in data using
        the ModelParameterDictionary and sets the random variables
        according to their respective distribution
        '''
        
        self.shape_parameter = 0.0
        '''
        Shape parameter describes the skew of the Weibull distribution. 
        If shape < 3.5, data skews left.
        If shape == 3.5, data is normal.
        If shape > 3.5, data skews right.
        '''
        
        self.scale_paramter = 0.0
        '''
        Represents the peak of the Weibull function, located at the 63.5% value of the CDF.
        If unknown, it can be found using the mean and the get_scale_parameter() method.
        '''
        
        self.mean_fire_recurrence = 0.0
        '''
        Average fire recurrence for a given area, elevation, vegetation type, etc.
        '''
        
        self.total_run_time = 0
        self.delta_t = 0
        '''Initial run time values
        to be read in.
        '''
        
        self.time_to_next_fire = 0.0
        '''
        Initial value which is replaced with values generated from the 
        random.weibullvariate() function based on the scale and shape parameters
        '''
        
    def initialize(self, input_file = None):
        MPD = ModelParameterDictionary()
        
        '''
        We import methods from ModelParameterDictionary
        to read the parameters from the input file.
        
        If the scale parameter is unknown, it can be found using the mean
        fire recurrence value, which MUST be known or estimated to run the
        get_scale_parameter() method.
        
        '''
        if input_file is None:
            input_file = _DEFAULT_INPUT_FILE
        MPD.read_from_file(input_file)
        
        self.shape_parameter = MPD.read_float('SHAPE_PARAMETER')
        self.scale_parameter = MPD.read_float('SCALE_PARAMETER')
        self.mean_fire_recurrence = MPD.read_float('MEAN_FIRE_RECURRENCE')
        self.total_run_time = MPD.read_float('RUN_TIME')
        self.delta_t = MPD.read_int('DELTA_T')


    def get_scale_parameter(self):
        
        ''' If the scale factor is unknown, we can draw it from the mean
        fire recurrence interval, given the following equation:
            
            This ONLY works if we have the shape parameter. Generally, the shape
            parameter will be greater than 3.5, creating a distribution that is skewed
            to the higher values (skewed to the right). 
            
        Mean_fire_recurrence = scale_parameter*(gamma_function(1+(1/shape)))
        '''
        
        if self.scale_parameter == 0.0:   
            shape_in_gamma_func = float(1+(1/self.shape_parameter))
            gamma_func = special.gamma(shape_in_gamma_func)
            self.scale_parameter = (self.mean_fire_recurrence/gamma_func)
            return self.scale_parameter
        else:
            return self.scale_parameter
            
    def generate_fire_recurrence(self):
        
        ''' Finds the time to next fire (fire recurrence) based on the scale parameter (63.5% of
        fire Weibull distribution) and the shape parameter (describes the skew of the histogram, shape = 3.5
        represents a normal distribution).
        
        Rounds the time to next fire to 4 significant figures, for neatness.
        '''
        
        self.time_to_next_fire = round(weibullvariate(self.scale_parameter, self.shape_parameter),2)
        return self.time_to_next_fire
        
    def generate_fire_time_series(self):
        
        '''
        Allows for a series of fire events to be generated given 
        a total time. 
        
        Created for situations where total run time is definite, and number
        of fires can change across different runs. 
        '''
        self.fire_events =[]
        #first_event = 0.0
        event = self.generate_fire_recurrence()
        end_event = event + 365.0
        self.fire_events.append([event, end_event])
        t = 0
        i = 0
        while t <= self.total_run_time:
            fire = self.generate_fire_recurrence()
            start_fire = self.fire_events[i][0] + (fire)
            end_fire = start_fire + (365.0)       
            self.fire_events.append([start_fire, end_fire])
            t += end_fire
            i+=1

        
    def update(self):
        
        '''
        Update function allows us to update the value of 'time_to_next_fire' by
        re-calling the generate_fire_reccurence() function.
        
        Created for instances when a definite number of fires need to
        be generated.
        '''
        
        self.time_to_next_fire = self.generate_fire_recurrence()
        return self.time_to_next_fire






#! /usr/env/python

"""
test_script_for_fastscape_stream_power.py: 
    
Tests and illustrates use of route_flow_dn component.
"""

from landlab import RasterModelGrid
from landlab.components.flow_routing.route_flow_dn import FlowRouter
from landlab.io import read_esri_ascii
from landlab.plot.imshow import imshow_node_grid
import os
import pylab
from numpy import ones, size

dem_name = '../../io/tests/data/west_bijou_gully.asc'
outlet_row = 82
outlet_column = 38

# Read in a DEM and set its boundaries (we'll try this one later)
DATA_FILE = os.path.join(os.path.dirname(__file__), dem_name)
(dem_grid, z) = read_esri_ascii(DATA_FILE)
dem_grid.set_nodata_nodes_to_inactive(z, 0) # set nodata nodes to inactive bounds
outlet_node = dem_grid.grid_coords_to_node_id(outlet_row, outlet_column)

grid = RasterModelGrid(4, 5, 1.0)
grid.set_inactive_boundaries(True, False, True, True)
z = grid.add_zeros('node', 'Land_Surface__Elevation')
z[6] = 4.5
z[7] = 3.
z[8] = 1.
z[11] = 4.
z[12] = 2.8
z[13] = 2.


# Get array of interior (active) node IDs
interior_nodes = grid.get_active_cell_node_ids()

# Route flow
flow_router = FlowRouter(grid)
r, a, q, ss, s, rl = flow_router.route_flow(z)

for i in range(grid.number_of_nodes):
    print i, grid.node_x[i], grid.node_y[i], z[i], grid.node_status[i], \
          r[i], a[i], q[i], ss[i], rl[i]

# Let's take a look for debugging
#print 'node  receiver  flow_link'
#for i in interior_nodes:
#    print i, r[i], rl[i]

# Calculate lengths of flow links
flow_link_length = ones(size(z))
flow_link_length[interior_nodes] = grid.link_length[rl[interior_nodes]]
print 'fll:', flow_link_length

# Get a 2D array version of the elevations
ar = grid.node_vector_to_raster(a)

numcols = grid.number_of_node_columns
numrows = grid.number_of_node_rows
dx = grid.dx

# Create a shaded image
pylab.close()  # clear any pre-existing plot
im = pylab.imshow(ar, cmap=pylab.cm.RdBu, extent=[0,numcols*dx,0,numrows*dx])
# add contour lines with labels
cset = pylab.contour(ar, extent=[0,numcols*dx,numrows*dx,0])
pylab.clabel(cset, inline=True, fmt='%1.1f', fontsize=10)
    
# add a color bar on the side
cb = pylab.colorbar(im)
cb.set_label('Drainage area, sq meters')
    
# add a title and axis labels
pylab.title('DEM')
pylab.xlabel('Distance (m)')
pylab.ylabel('Distance (m)')

# Display the plot
pylab.show()


# HERE IS THE STREAM POWER PART

import numpy

K = 0.00001
dt = 100.0
m = 0.5
num_time_steps = 1

# Derived parameters
Kdt = K*dt  # saves lots of multiplying later

# Time loop
for t in range(num_time_steps):
    
    # Update baselevel nodes if applicable
    # (to be added)

    # Calculate A^m and K A^m dt / L using Numpy array operations, for speed
    # (TODO: flow router should return an array of flow-link lengths)
    Am = numpy.power(a, m)
    alpha = Kdt*Am/flow_link_length

    # Loop over nodes from downstream to upstream, updating elevations using 
    # analytical solution (implicit)
    for i in s:  # for each node ID, in order from downstream to upstream
        
        j = r[i]   # receiver (downstream node) of i
        print i, j
        if i != j:  # if sender and receiver are same, it's a boundary node
            z[i] += alpha[i]*z[j]/(1.0+alpha[i])




#! /usr/env/python
"""

2D numerical model of shallow-water flow over topography read from a DEM, using
the Bates et al. (2010) algorithm for storage-cell inundation modeling.

Last updated GT Oct 2013

"""

from landlab.io import read_esri_ascii
import time
import os
import pylab
import numpy as np


def main():
    """
    In this simple tutorial example, the main function does all the work: 
    it sets the parameter values, creates and initializes a grid, sets up 
    the state variables, runs the main loop, and cleans up.
    """
    
    # INITIALIZE
    
    # User-defined parameter values
    dem_name = 'ExampleDEM/west_bijou_gully.asc'
    outlet_row = 82
    outlet_column = 38
    next_to_outlet_row = 81
    next_to_outlet_column = 38
    n = 0.06              # roughness coefficient (Manning's n)
    h_init = 0.001        # initial thin layer of water (m)
    g = 9.8               # gravitational acceleration (m/s2)
    alpha = 0.2           # time-step factor (ND; from Bates et al., 2010)
    run_time = 2400       # duration of run, seconds
    rainfall_mmhr = 100   # rainfall rate, in mm/hr
    rain_duration = 15*60 # rainfall duration, in seconds
    
    # Derived parameters
    rainfall_rate = (rainfall_mmhr/1000.)/3600.  # rainfall in m/s
    ten_thirds = 10./3.   # pre-calculate 10/3 for speed
    elapsed_time = 0.0    # total time in simulation
    report_interval = 5.  # interval to report progress (seconds)
    next_report = time.time()+report_interval   # next time to report progress
    DATA_FILE = os.path.join(os.path.dirname(__file__), dem_name)
    
    # Create and initialize a raster model grid by reading a DEM
    print('Reading data from "'+str(DATA_FILE)+'"')
    (mg, z) = read_esri_ascii(DATA_FILE)
    print('DEM has ' + str(mg.number_of_node_rows) + ' rows, ' +
          str(mg.number_of_node_columns) + ' columns, and cell size ' + str(mg.dx))
    
    # Modify the grid DEM to set all nodata nodes to inactive boundaries
    mg.set_nodata_nodes_to_inactive(z, 0) # set nodata nodes to inactive bounds
    
    # Set the open boundary (outlet) cell. We want to remember the ID of the 
    # outlet node and the ID of the interior node adjacent to it. We'll make
    # the outlet node an open boundary.
    outlet_node = mg.grid_coords_to_node_id(outlet_row, outlet_column)
    node_next_to_outlet = mg.grid_coords_to_node_id(next_to_outlet_row, 
                                                    next_to_outlet_column)
    mg.set_fixed_value_boundaries(outlet_node)

    # Set up state variables
    h = mg.create_node_array_zeros() + h_init     # water depth (m)
    q = mg.create_active_link_array_zeros()       # unit discharge (m2/s)
    
    # Get a list of the interior cells
    interior_cells = mg.get_active_cell_node_ids()
    
    # To track discharge at the outlet through time, we create initially empty
    # lists for time and outlet discharge.
    q_outlet = []
    t = []
    q_outlet.append(0.)
    t.append(0.)
    outlet_link = mg.get_active_link_connecting_node_pair(outlet_node, 
                                                          node_next_to_outlet)
    
    # Display a message
    print( 'Running ...' )
    start_time = time.time()

    # RUN
    
    # Main loop
    while elapsed_time < run_time:
        
        # Report progress
        if time.time()>=next_report:
            print('Time = '+str(elapsed_time)+' ('
                  +str(100.*elapsed_time/run_time)+'%)')
            next_report += report_interval
        
        # Calculate time-step size for this iteration (Bates et al., eq 14)
        dtmax = alpha*mg.dx/np.sqrt(g*np.amax(h))
        
        # Calculate the effective flow depth at active links. Bates et al. 2010
        # recommend using the difference between the highest water-surface
        # and the highest bed elevation between each pair of cells.
        zmax = mg.max_of_link_end_node_values(z)
        w = h+z   # water-surface height
        wmax = mg.max_of_link_end_node_values(w)
        hflow = wmax - zmax
        
        # Calculate water-surface slopes
        water_surface_slope = mg.calculate_gradients_at_active_links(w)
       
        # Calculate the unit discharges (Bates et al., eq 11)
        q = (q-g*hflow*dtmax*water_surface_slope)/ \
            (1.+g*hflow*dtmax*n*n*abs(q)/(hflow**ten_thirds))
        
        # Calculate water-flux divergence at nodes
        dqds = mg.calculate_flux_divergence_at_nodes(q)
        
        # Update rainfall rate
        if elapsed_time > rain_duration:
            rainfall_rate = 0.
        
        # Calculate rate of change of water depth
        dhdt = rainfall_rate-dqds
        
        # Second time-step limiter (experimental): make sure you don't allow
        # water-depth to go negative
        if np.amin(dhdt) < 0.:
            shallowing_locations = np.where(dhdt<0.)
            time_to_drain = -h[shallowing_locations]/dhdt[shallowing_locations]
            dtmax2 = alpha*np.amin(time_to_drain)
            dt = np.min([dtmax, dtmax2])
        else:
            dt = dtmax
        
        # Update the water-depth field
        h[interior_cells] = h[interior_cells] + dhdt[interior_cells]*dt
        h[outlet_node] = h[node_next_to_outlet]
        
        # Update current time
        elapsed_time += dt
        
        # Remember discharge and time
        t.append(elapsed_time)
        q_outlet.append(q[outlet_link])
        
      
    # FINALIZE
    
    # Set the elevations of the nodata cells to the minimum active cell
    # elevation (convenient for plotting)
    z[np.where(z<=0.)] = 9999            # temporarily change their elevs ...
    zmin = np.amin(z)                    # ... so we can find the minimum ...
    z[np.where(z==9999)] = zmin          # ... and assign them this value.

    # Get a 2D array version of the water depths and elevations
    hr = mg.node_vector_to_raster(h)
    zr = mg.node_vector_to_raster(z)
    
    # Clear previous plots
    pylab.figure(1)
    pylab.close()
    pylab.figure(2)
    pylab.close()
    
    # Plot discharge vs. time
    pylab.figure(1)
    pylab.plot(np.array(t), np.array(q_outlet)*mg.dx)
    pylab.xlabel('Time (s)')
    pylab.ylabel('Q (m3/s)')
    pylab.title('Outlet discharge')
    
    # Plot topography
    pylab.figure(2)
    pylab.subplot(121)
    im = pylab.imshow(zr, cmap=pylab.cm.RdBu,
                      extent=[0, mg.number_of_node_columns * mg.dx,
                              0, mg.number_of_node_rows * mg.dx])
    cb = pylab.colorbar(im)
    cb.set_label('Elevation (m)', fontsize=12)
    pylab.title('Topography')
    
    # Plot water depth
    pylab.subplot(122)
    im2 = pylab.imshow(hr, cmap=pylab.cm.RdBu,
                       extent=[0, mg.number_of_node_columns * mg.dx,
                               0, mg.number_of_node_rows * mg.dx])
    pylab.clim(0, 0.25)
    cb = pylab.colorbar(im2)
    cb.set_label('Water depth (m)', fontsize=12)
    pylab.title('Water depth')
    
    # Display the plots
    pylab.show()
    print('Done.')
    print('Total run time = '+str(time.time()-start_time)+' seconds.')


if __name__ == "__main__":
    main()



import numpy as np
from numpy.testing import assert_array_equal
from nose import with_setup
from nose.tools import assert_is

from landlab import RasterModelGrid


_GRIDS = {}

def setup_grids():
    _GRIDS.update({
        'unit': RasterModelGrid(4, 5),
        'non_unit': RasterModelGrid(4, 5, 2.),
    })


@with_setup(setup_grids)
def test_unit_spacing():
    rmg, values_at_nodes = _GRIDS['unit'], np.arange(20)
    grads = rmg.calculate_gradients_at_active_links(values_at_nodes)

    assert_array_equal(grads, np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                        1, 1, 1, 1, 1, 1, 1, 1]))

    diffs = rmg.calculate_diff_at_active_links(values_at_nodes)
    assert_array_equal(grads, diffs)
    
    
@with_setup(setup_grids)
def test_non_unit_spacing():
    rmg, values_at_nodes = _GRIDS['non_unit'], np.arange(20)

    grads = rmg.calculate_gradients_at_active_links(values_at_nodes)
    assert_array_equal(grads, (1. / rmg.node_spacing) *
                       np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                 1, 1, 1, 1, 1, 1, 1, 1]))
    diffs = rmg.calculate_diff_at_active_links(values_at_nodes)
    assert_array_equal(grads, (1. / rmg.node_spacing) * diffs)


@with_setup(setup_grids)
def test_out_array():
    rmg, values_at_nodes = _GRIDS['non_unit'], np.arange(20)

    output_array = np.empty(17)
    rtn_array = rmg.calculate_gradients_at_active_links(values_at_nodes,
                                                        out=output_array)
    assert_array_equal(rtn_array, np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                            1, 1, 1, 1, 1, 1, 1, 1]) / rmg.node_spacing)
    assert_is(rtn_array, output_array)
    
    
@with_setup(setup_grids)
def test_diff_out_array():
    rmg = RasterModelGrid(4, 5)
    values = np.arange(20)
    diff = np.empty(17)
    rtn_diff = rmg.calculate_diff_at_active_links(values, out=diff)
    assert_array_equal(
        diff,
        np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                  1, 1, 1, 1, 1, 1, 1, 1]))
    assert_is(rtn_diff, diff)



########## soil_moisture.py ###############
##
##  This component calculates and updates soil moisture after each storm.
##  Soil moisture is represented as a single bucket. Rainfall depth fills the bucket
##  and the soil moisture decays as a result of leakage and ET following the analytical solution of
##  Laio et al., (2001) 
##  This component can operate on any raster grid
##  Input file is named soilmoisture_input.txt and is temporarily placed under landlab.components.
##  All input values can be altered using set_*** functions
##  All output arrays can be retrieved using get_*** functions
##
##  Storms are considered to be instantaneous events.
##  Storm duration, depth and interstorm duration are obtained from RainfallDriver.py
##  Storm depth is obtained in mm and storm duration and interstorm duration are obtained in hours
##
##  Sai Nudurupati and E.I. 11/17/2013.
###########################################

import os
from numpy import *
from math import *
import landlab
from landlab import RasterModelGrid
from landlab.model_parameter_dictionary import ModelParameterDictionary
from landlab.components.uniform_precip.generate_uniform_precip import PrecipitationDistribution
from landlab.plot import imshow_grid, imshow_active_cells
from matplotlib.pyplot import *


_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'soilmoisture_input.in')

class SoilMoisture():

    def __init__( self ):

        """ vegcover - vegetation cover fraction """
        self._vegcover = 0.0
        """ interception_cap - Maximum interception capacity """
        self._interception_cap = 0.0
        """ zr - root depth """
        self._zr = 0.0
        """ runon - runon from upstream """
        self._runon = 0.0
        """ PET - potential evapotranspiration """
        self._PET = 0.0
        """ fbare - scaling coefficient for bare soil Potential Evapotranspiration """
        self._fbare = 0.0
        """ Ib - Bare soil infiltration capacity [mm/h] """
        self._soil_Ib = 0.0
        """ Iv - Vegetated surface infiltration capacity [mm/h] """
        self._soil_Iv = 0.0
        """ Ew - Residual Evaporation after wilting """
        self._soil_Ew = 0.0
        """ pc - porosity """
        self._soil_pc = 0.0
        """ fc - field capacity """
        self._soil_fc = 0.0
        """ sc - stomata closure moisture fraction """
        self._soil_sc = 0.0
        """ wp - wilting point """ 
        self._soil_wp = 0.0
        """ hgw - Hygroscopic water """
        self._soil_hgw = 0.0
        """ beta - Deep percolation rate constant """
        self._soil_beta = 0.0
     
        
        """ initialize """
        P = 0
        Tb = 0
        Tr = 0
        ETA = 0.0
        Water_Stress = 0.0
        
        """ sini - soil moisture with storm at the end of time intervals """
        sini = 0.0
        """ s - final soil moisture """
        s = 0.0
        """ Dd - Drainage from rootzone """
        Dd = 0.0
        """ Runoff - runoff after storm """
        Runoff = 0.0
        
        """ Iterations - Number of iterations/storms """
        self._iterate_storm = 0
        """ so - Soil moisture of the ground before storm """
        so = 0.0

        """ Initialize output variables  """
        
        self._Peff = 0.0        
        self._time = 0.0
        
       
    def initialize( self ):

        """ Read Input File """        
        MPD = ModelParameterDictionary()
        MPD.read_from_file(_DEFAULT_INPUT_FILE)
        
        """ Read Input Parameters """        
        self._iterate_storm = MPD.read_int( 'N_STORMS' )
        self._vegcover = MPD.read_float( 'VEG_COV' )
        self._interception_cap = MPD.read_float( 'INTERCEPT_CAP' )
        self._zr = MPD.read_float( 'ZR' )
        self._runon = MPD.read_float( 'RUNON' )
        self._fbare = MPD.read_float( 'F_BARE' )
        self._soil_Ib = MPD.read_float( 'I_B' )
        self._soil_Iv = MPD.read_float( 'I_V' )
        self._soil_Ew = MPD.read_float( 'EW' )
        self._soil_pc = MPD.read_float( 'PC' )
        self._soil_fc = MPD.read_float( 'FC' )
        self._soil_sc = MPD.read_float( 'SC' )
        self._soil_wp = MPD.read_float( 'WP' )
        self._soil_hgw = MPD.read_float( 'HGW' )
        self._soil_beta = MPD.read_float( 'BETA' )        
        

    def update( self, P, Tb, Tr, VT, SO ):

        """
            Reading in 'P' storm depth, 'Tb' Interstorm duration, 'Tr' Storm
            duration, 'fveg' Vegetation factor, 'SO' Initial Soil Moisture
        """
        
        """
            Initializing parameters that are constant in space for a given Storm
        """
        
        #V = fveg     # total veg cover used for interception
        fbare = self._fbare
        ZR = self._zr
        pc = self._soil_pc
        fc = self._soil_fc
        sc = self._soil_sc
        wp = self._soil_wp
        hgw = self._soil_hgw
        beta = self._soil_beta

        """
            Calculating parameters that are constant over space
        """
        
        #Inf_cap = self._soil_Ib*(1-V) + self._soil_Iv*V # Infiltration capacity
        #Int_cap = min(V*self._interception_cap, P)  # Interception capacity
        #Peff = max(P-Int_cap, 0.0)             # Effective precipitation depth
        #mu = (Int_cap/1000.0)/(pc*ZR*(exp(beta*(1-fc))-1))        
        
        """ 
            Initializing variables for spatial run for a given storm
        """
        
        (mm,nn) = SO.shape
        
        self._ETA = zeros((SO.shape))
        self._S = zeros((SO.shape))
        self._D = zeros((SO.shape))
        self._Ro = zeros((SO.shape))
        self._WS = zeros((SO.shape))
        self._mu = zeros((SO.shape))  # for debugging
        self._nu = zeros((SO.shape))  # for debugging
        self._munu = zeros((SO.shape)) # for debugging

        """
            Looping over space for the time period (Tr+Tb)
        """
        
        """ Rows """
                
        for i in range(0,mm):
            
            """ Columns """    
            
            for j in range(0,nn):

                V = VT[i][j]
                Inf_cap = self._soil_Ib*(1-V) + self._soil_Iv*V # Infiltration capacity
                Int_cap = min(V*self._interception_cap, P)  # Interception capacity
                Peff = max(P-Int_cap, 0.0)             # Effective precipitation depth
                mu = (Int_cap/1000.0)/(pc*ZR*(exp(beta*(1-fc))-1)) 
                
                """
                    Calculating parameters that are variable over space
                """
                
                pet = self._PET[i][j]           # Potential Evapotranspiration
                Ep = max((pet*V+fbare*pet*(1-V)) - Int_cap, 0.0)  #
                nu = ((Ep/24.0)/1000.0)/(pc*ZR) # Loss function parameter
                nuw = ((Ep*0.1/24)/1000.0)/(pc*ZR) # Loss function parameter
                so = SO[i][j]               # Initial Soil Moisture
                sini = so + (Peff/(pc*ZR*1000.0))+self._runon 
                self._mu[i][j] = mu   # for debugging
                self._nu[i][j] = nu   # for debugging 
                self._munu[i][j] = mu-nu # for debugging  
        
                """
                    Evaluating Runoff and accordingly adjusting initial soil
                    moisture
                """          
                      
                if sini>1:
                    runoff = (sini-1)*pc*ZR*1000
                    sini = 1
                else:
                    runoff = 0

                """
                    Evaluating the current 'regime' of soil moisture (Laio et al.
                    Fig. 5) and calculating the evolution time and subsequently
                    calculating final soil moisture, drainage and actual 
                    evapotranspiration (Analytical solution, Laio et al.)
                """
        
                if sini>=fc:
                    tfc = (1.0/(beta*(mu-nu)))*(beta*(fc-sini)+                \
                            log((nu-mu+mu*exp(beta*(sini-fc)))/nu))
                    if nu == 0:
                        print 'nu is zero', nu
                        print 'Veg Cover', V
                        print 'Ep = ', Ep
                        print 'pet = ', pet
                    tsc = ((fc-sc)/nu)+tfc
                    twp = ((sc-wp)/(nu-nuw))*log(nu/nuw)+tsc
        
                    if Tb<tfc:
                        s = abs(sini-(1/beta)*log(((nu-mu+mu*                  \
                             exp(beta*(sini-fc)))*exp(beta*(nu-mu)*Tb)         \
                             -mu*exp(beta*(sini-fc)))/(nu-mu)))

                        Dd = ((pc*ZR*1000)*(sini-s))-(Tb*(Ep/24))
                        ETA = (Tb*(Ep/24))
        
                    elif Tb>=tfc and Tb<tsc:
                        s = fc-(nu*(Tb-tfc))
                        Dd = ((pc*ZR*1000)*(sini-fc))-((tfc)*(Ep/24))
                        ETA = (Tb*(Ep/24))
        
                    elif Tb>=tsc and Tb<twp:
                        s = wp+(sc-wp)*((nu/(nu-nuw))*exp((-1)*((nu-nuw)/(sc-wp))*(Tb-tsc))-(nuw/(nu-nuw)))
                        Dd = ((pc*ZR*1000)*(sini-fc))-(tfc*Ep/24)
                        ETA = (1000*ZR*pc*(sini-s))-Dd
        
                    else:
                        s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*max(Tb-twp,0))
                        Dd = ((pc*ZR*1000)*(sini-fc))-(tfc*Ep/24)
                        ETA = (1000*ZR*pc*(sini-s))-Dd
        
                elif sini<fc and sini>=sc:
                    tfc = 0
                    tsc = (sini-sc)/nu
                    twp = ((sc-wp)/(nu-nuw))*log(nu/nuw)+tsc
        
                    if Tb<tsc:
                        s = sini - nu*Tb
                        Dd = 0
                        ETA = 1000*ZR*pc*(sini-s)
        
                    elif Tb>=tsc and Tb<twp:
                        s = wp+(sc-wp)*((nu/(nu-nuw))*exp((-1)*((nu-nuw)/(sc-wp))*(Tb-tsc))-(nuw/(nu-nuw)))
                        Dd = 0
                        ETA = (1000*ZR*pc*(sini-s))
        
                    else:
                        s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*(Tb-twp))
                        Dd = 0
                        ETA = (1000*ZR*pc*(sini-s))
                        
                elif sini<sc and sini>=wp:
                    tfc = 0
                    tsc = 0
                    twp = ((sc-wp)/(nu-nuw))*log(1+(nu-nuw)*(sini-wp)/(nuw*(sc-wp)))
        
                    if Tb<twp:
                        s = wp+((sc-wp)/(nu-nuw))*((exp((-1)*((nu-nuw)/(sc-wp))*Tb))*(nuw+((nu-nuw)/(sc-wp))*(sini-wp))-nuw)
                        Dd = 0
                        ETA = (1000*ZR*pc*(sini-s))
        
                    else:
                        s = hgw+(wp-hgw)*exp((-1)*(nuw/(wp-hgw))*(Tb-twp))
                        Dd = 0
                        ETA = (1000*ZR*pc*(sini-s))
        
                else:
                    tfc = 0
                    tsc = 0
                    twp = 0
        
                    s = hgw+(sini-hgw)*exp((-1)*(nuw/(wp-hgw))*Tb)
                    Dd = 0
                    ETA = (1000*ZR*pc*(sini-s))
        
                Water_Stress = min(max(pow(((sc - ((s+sini)/2)) / (sc - wp)),4),0.0),1.0) 
            
                """ varaibles for get/set functions """
                self._Peff = Peff
                self._ETA[i][j] = ETA
                self._S[i][j] = s
                self._D[i][j] = Dd
                self._Ro[i][j] = runoff                
                self._WS[i][j] = Water_Stress
                #self._so[i][j] = s 
        self._time = self._time + (Tb + Tr)/(24*365.4)  

    """ Return Effective precipittion mm """
    def get_Peff( self ):
        return self._Peff
    
    """ Return Current Time """  
    def get_time( self ):
        return self._time
 
    """ Return Interstorm ET mm """
    def get_ET( self ):
        return self._ETA

    """ Return Soil Moisture  """
    def get_S( self ):
        return self._S

    """ Return Drainage from Root Zone mm """
    def get_D( self ):
        return self._D

    """ Return Runoff mm """
    def get_Ro( self ):
        return self._Ro
    
    """ Return Water Stress """
    def get_WaterStress( self ):
        return self._WS

    """ Set Number of Storms """
    def set_strms( self, strms ):
        self._iterate_storm = strms        
        
    """ Set Potential Evapotranspiration Rate """
    def set_PET( self, PET ):
        self._PET = PET


    def plott( self, P_, S_, ETA_, D_, Ro_, Time_, RMG ):

        figure(1)
        plot( Time_, P_ )
        ylabel( 'Storm Depth [mm]' )
        title( 'Storm generation' )
        xlabel( 'Time:Year' )
        savefig('StormDepth')        

        figure(2)
        imshow_grid(RMG, S_, values_at = 'cell')
        savefig('CurrentSM_State') 

        show()


import numpy as np
from pylab import show, imshow, colorbar, plot
import landlab.grid.raster as raster
from landlab.components.flow_routing import flow_routing_D8
from landlab.components.flow_accum import flow_accumulation
from landlab.plot.imshow import imshow_grid
reload(flow_routing_D8)
reload(flow_accumulation)
reload(raster)

class data(object):
    '''
        This is where all the whole-grid data lives, as arrays over the various elements of the grid.
        '''
    #Data goes here!!!
    def __init__(self, grid):
        self.elev = grid.zeros(centering='node') #some data

#Make grid, set the elevs
mg = raster.RasterModelGrid(5, 5, 1.)
mg.set_inactive_boundaries(True, True, False, True)
vectors = data(mg)

vectors.elev = np.array([10.,10.,0.,10.,10.,10.,8.,8.,9.,10.,10.,4.,6.,7.,10.,10.,1.,3.,8.,10.,10.,0.,10.,10.,10.])

print vectors.elev.reshape((5,5))
print np.array(range(25)).reshape((5,5))
#print mg.node_status.reshape((5,5))

#imshow_grid(mg,vectors.elev)
#plot()

#Route the flow
dirs = flow_routing_D8.RouteFlowD8(len(vectors.elev))
vectors.flowdirs, vectors.node_max_gradients = dirs.calc_flowdirs(mg, vectors.elev)

#print vectors.flowdirs.shape
#print vectors.flowdirs.reshape((5,5))

#Accumulate the flow
accum = flow_accumulation.AccumFlow(mg, vectors)
#vectors.flowacc = accum.calc_flowacc(mg, vectors)
accum.calc_flowacc(mg, vectors)

print vectors.flowacc.shape
print vectors.flowacc.reshape((5,5))



#import os
from landlab.components.fire_generator.generate_fire import FireGenerator
from matplotlib import pyplot as plt
#filename = '/Users/Jordan/landlab/landlab/components/uniform_precip/examples/fireraininput.txt'



print 'Initializing the FireGenerator() class from fire_generator.py'
FG = FireGenerator()
FG.initialize()

print 'Finding unknown scale parameter given the mean fire recurrence value'
FG.get_scale_parameter()

print 'Finding a time series based on the Weibull distribution'
FG.generate_fire_time_series()

print 'Sorting the fire series to test the distribution...'
FG.fires.sort()

print 'Plotting the histogram to show the distribution.'
ymax = int(FG.total_run_time)+1
yaxis = range(0, ymax)
plt.plot(FG.fires, yaxis)
plt.hist(FG.fires)
plt.show()



########## radiation.py ###############
##
##  This code calculates incident Shortwave radiation on a 
##  surface taking into account location, slope and aspect
##  of the surface.
##
##  ref: Javier H. Flores Cervantes et al. 2012 "A geomorphic .." Ecohydrol
##
## Written by Sai Nudurupati & Erkan Istanbulluoglu, 2013.
###########################################

import os
from numpy import *
from math import *
import landlab
from landlab import RasterModelGrid  # add landlab. as prefix if have landlab installed
from landlab import ModelParameterDictionary # add landlab. as prefix
from matplotlib.pyplot import *
from random import random
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import LinearLocator, FormatStrFormatter
from landlab.plot import imshow_grid, imshow_active_cells

# Input file 
_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                 'radiation_input.txt')


class Radiation():

    def __init__( self ):

        self._latitude = 0.0       # Latitude in deg
        self._t = 0.0              # Local Time
        self._dt = 0.0             # Local Time Step 
        self._beta = 0.0           # Aspect
        self._slope = 0.0          # Slope
        self._julian = 0.0         # Julian Day
        self._current_dt = 0.0     # Time Step
        self._current_time = 0.0   # Time
        self._phi = 0.0       # Latitude in Radians
        self._delta = 0.0     # Declination Angle of Sun
        self._tau = 0.0       # Hour Angle
        self._alpha = 0.0     # Solar Altitude
        self._phisun = 0.0    # Sun's Azhimuth
        self._flat = 0.0      # Flat Surface Reference Cos(Solar Incidence)
        #self._sig = 5.6e-8   # Stefan Boltzmann's Constant
        self._n = 2.0         # Clear Sky Turbidity Factor 
        self._m = 0.0         # Optical Air Mass
        self._N = 0.0         # Cloudiness
        self._A = 0.0         # Albedo
        self._Io = 1353.0     # Solar Constant
        
        
    def initialize( self ):
        
        MPD = ModelParameterDictionary()
        MPD.read_from_file( _DEFAULT_INPUT_FILE )

        # Reading Input Parameters
        self._N = MPD.read_float( 'CLOUDINESS' )        
        self._latitude = MPD.read_float( 'LATITUDE' )
        self._A = MPD.read_float( 'ALBEDO' )

        
    def update( self, RMG ):
        
        self._Si = RMG.empty(centering = 'cell')  # Cos(Solar Angle of Incidence)
        self._BETA = RMG.empty(centering = 'cell') # Aspect (Clckwise from N)
        self._Rs = RMG.empty(centering = 'cell')   # Incoming Shortwave Radn            
        self._slope_ = RMG.empty(centering = 'cell') # Slope - Rise/Run
        self._slope = RMG.empty(centering = 'cell')  # Slope - in Radians
        self._angles = RMG.empty(centering = 'cell') # Angles - Clckwise from N      
                       
        self._t = 12                                     # Assuming noon
        self._julian = floor( (self._current_time - floor( self._current_time))\
                                                        * 365 ) # Julian day

        print 'Julian Day = ', self._julian
                
        self._phi = pi/180.0 * self._latitude             # Latitude in Radians
        
        self._delta = 23.45 * pi/180.0 * cos(2*pi/365 * (172 - self._julian))  \
                                                            # Declination angle

        self._tau = (self._t + 12) * pi/12.0                   # Hour Angle

        self._alpha = asin(sin(self._delta) * sin(self._phi) + cos(self._delta)\
                        *cos(self._phi)*cos(self._tau))       # Solar Altitude

        if self._alpha <= 0.25*pi/180:          # If altitude is -ve, 
            self._alpha = 0.25*pi/180           # sun is beyond the horizon

        self._Rgl = (1 - self._A) * (1 - 0.65 * pow((self._N),2)) *            \
                 (self._Io*exp((-1) * self._n * (0.128 - 0.054 *               \
                    log10(1/sin(self._alpha)))*(1/sin(self._alpha))))
                    # Counting for Albedo, Cloudiness and Atmospheric turbidity

        self._phisun = atan(-sin(self._tau)/(tan(self._delta)*cos(self._phi)   \
                        - sin(self._phi)*cos(self._tau)))     # Sun's Azhimuth

        if ( self._phisun >= 0 and -sin(self._tau) <= 0 ):
            self._phisun = self._phisun + pi
            
        elif ( self._phisun <= 0 and -sin(self._tau) >= 0 ):
            self._phisun = self._phisun + pi
            

        self._flat = cos(atan(0)) * sin(self._alpha) + sin(atan(0)) *          \
                        cos(self._alpha) * cos(self._phisun - 0)
                                                       # flat surface reference 
             
        for i in range(0, RMG.number_of_cells):

            self._slope_[i], self._angles[i] =                                 \
                RMG.calculate_max_gradient_across_node( self._Z,               \
                  RMG.cell_node[i])

            self._slope[i] = atan(self._slope_[i])   
            self._BETA[i] = self._angles[i]*pi/180  
            
            self._Si[i] = (cos(self._slope[i]) * sin(self._alpha) +            \
                                sin(self._slope[i]) * cos(self._alpha)         \
                                  * cos(self._phisun - self._BETA[i]))
            
            if self._Si[i] <= 0:
                self._Si[i] = 0.0

            self._Rs[i] = self._Rgl * self._Si[i]    # Incoming Shortwave Radn
            

    def plott( self, RMG ):
        
        figure(1)        
        imshow_grid(RMG,self._Z, values_at = 'node')
        savefig('figure1')

        figure(2)
        self._slope_[self._slope_ >= 1] = 1
        imshow_active_cells(RMG, self._slope_, 'Slope', None, ('m','m'))
        savefig('figure2')

        figure(3)
        imshow_active_cells(RMG,self._Si,                                    \
                   'Cosine of Solar Angle of Incidence', None, ('m','m'))
        savefig('figure3') 
        
        figure(4)
        imshow_active_cells(RMG,self._BETA,                                    \
            'Aspect (clockwise from North)', 'radians', ('m','m'))
        savefig('figure4')
                
        figure(5)
        imshow_active_cells(RMG,self._Rs,                                    \
                   'Incoming Shortwave Radiation', 'W/m^2', ('m','m'))     
        savefig('figure5')
    
    def set_current_time( self, current_time ):
        
        self._current_time = current_time



#! /usr/bin/env python

import numpy

from landlab.grid.voronoi import VoronoiDelaunayGrid


class HexModelGrid(VoronoiDelaunayGrid):
    """
    This inherited class implements a regular 2D grid with hexagonal cells and
    triangular patches. It is a special type of VoronoiDelaunay grid in which
    the initial set of points is arranged in a triangular/hexagonal lattice.
    
    Examples:
        
        >>> hmg = HexModelGrid(3, 2, 1.0)
        >>> hmg.number_of_nodes
        7
    """
    
    def __init__(self, num_rows=0, base_num_cols=0, dx=1.0, **kwds):
        """
        Optionally takes numbers of rows and columns and cell size as
        inputs. If this are given, calls initialize() to set up the grid.
        
        """
        #print 'HexModelGrid.__init__'
        
        # Set number of nodes, and initialize if caller has given dimensions
        #self._num_nodes = num_rows * num_cols
        if num_rows * base_num_cols > 0:
            self._initialize(num_rows, base_num_cols, dx)
        super(HexModelGrid, self).__init__(**kwds)

    def _initialize( self, num_rows, base_num_cols, dx ):
        """
        Sets up a num_rows by num_cols grid with cell spacing dx and
        (by default) regular boundaries (that is, all perimeter cells are
        boundaries and all interior cells are active).

        To be consistent with unstructured grids, the hex grid is
        managed not as a 2D array but rather as a set of arrays that
        describe connectivity information between nodes, links, cells, faces,
        patches, corners, and junctions.
        """
        if self.DEBUG_TRACK_METHODS:
            print 'HexModelGrid._initialize('+str(num_rows)+', ' \
                   +str(base_num_cols)+', '+str(dx)+')'
        
        [pts, self._num_nodes] = self.make_hex_points(num_rows, base_num_cols, dx)
        super(HexModelGrid, self)._initialize(pts[:,0], pts[:,1])
        

    def make_hex_points(self, num_rows, base_num_cols, dxh):
        """
        Creates and returns a set of (x,y) points in a staggered grid in which the 
        points represent the centers of regular hexagonal cells, and the points
        could be connected to form equilateral triangles. The overall shape of the
        lattice is hexagonal.
        
        Inputs: num_rows = number of rows in lattice
                base_num_cols = number of columns in the bottom and top rows
                                (middle rows have more)
                dxh = horizontal and diagonal spacing between points
                
        Return: 2D numpy array containing point (x,y) coordinates, and total number
                of points.
                
        Example:
            
            >>> hmg = HexModelGrid()
            >>> [p, npt] = hmg.make_hex_points(3, 2, 1.0)
            >>> npt
            7
            >>> p[1,:]
            array([ 1.,  0.])
            >>> p[:3,0]
            array([ 0. ,  1. , -0.5])
        """

        dxv = dxh*numpy.sqrt(3.)/2.
        half_dxh = dxh/2.

        if numpy.mod(num_rows, 2)==0:  # even number of rows
            npts = num_rows*base_num_cols+(num_rows*num_rows)/4
            #print 'even # rows, npts=', npts
        else:  # odd number of rows
            npts = num_rows*base_num_cols + ((num_rows-1)/2)*((num_rows-1)/2)
            #print 'odd # rows, npts=', npts
        pts = numpy.zeros((npts,2))
        middle_row = num_rows/2
        extra_cols = 0
        xshift = 0.
        i=0
        for r in range(num_rows):
            for c in range(base_num_cols+extra_cols):
                pts[i,0] = c*dxh+xshift
                pts[i,1] = r*dxv
                i += 1
            if r<middle_row:
                extra_cols += 1
            else:
                extra_cols -= 1
            xshift = -half_dxh*extra_cols
        
        return pts, npts


def from_dict(param_dict):
    """
    Create a HexModelGrid from the dictionary-like object, *param_dict*.
    Required keys of the dictionary are NUM_ROWS, NUM_COLS. Raises a KeyError
    if either of these are missing.  If GRID_SPACING is given, use it as the
    HexModelGrid *dx* parameter, otherwise default to unit spacing.
    """
    # Read and create a basic HexModelGrid
    try:
        n_rows = int(param_dict['NUM_ROWS'])
        n_cols = int(param_dict['NUM_COLS'])
        dx = float(param_dict.get('GRID_SPACING', 1.))
    except KeyError as e:
        raise
    except ValueError as e:
        raise
    else:
        hg = HexModelGrid(n_rows, n_cols, dx)
        
    return hg
